<?php

/**
 * @author {@link mailto:corey@onpubco.com Corey HM Taylor}
 * @copyright Onpub (TM) Copyright (C) 2007, Onpubco
 * {@link http://onpubco.com/}
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * Version 2
 * @package onpubapi
 * @version 0.8.3
 */
class OnpubAAMaps
{
    private $pdo;
    /**
     * @var bool
     */
    public $enableTransactions;

    /**
     * @param PDO $pdo A {@link PHP_MANUAL#function.pdo-construct PHP Data Object}
     * @param bool $enableTransactions
     */
    function __construct( PDO $pdo, $enableTransactions = true )
    {
        $this->pdo = $pdo;
        $this->enableTransactions = $enableTransactions;
        $this->pdo->setAttribute( PDO::ATTR_EMULATE_PREPARES, true );
    }

    /**
     * @param int $articleID Delete the map(s) with articleID.
     * @param int $authorID Delete the map(s) with authorID.
     * @return int The number of maps deleted, 0 if no maps were deleted.
     */
    public function delete( $articleID, $authorID )
    {
        if ( $articleID && $authorID ) {
            $stmt = $this->pdo->prepare( "DELETE FROM OnpubAAMaps WHERE articleID = :articleID AND authorID = :authorID" );
            OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );
            $stmt->bindParam( ':articleID', $articleID );
            $stmt->bindParam( ':authorID', $authorID );
        }

        if ( $articleID && !$authorID ) {
            $stmt = $this->pdo->prepare( "DELETE FROM OnpubAAMaps WHERE articleID = :articleID" );
            OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );
            $stmt->bindParam( ':articleID', $articleID );
        }

        if ( !$articleID && $authorID ) {
            $stmt = $this->pdo->prepare( "DELETE FROM OnpubAAMaps WHERE authorID = :authorID" );
            OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );
            $stmt->bindParam( ':authorID', $authorID );
        }

        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, false, $stmt->errorInfo() );

        return $stmt->rowCount();
    }

    /**
     * @param OnpubQueryOptions $queryOptions Database query options.
     * @return array An array of {@link OnpubAAMap} objects.
     */
    public function select( OnpubQueryOptions $queryOptions )
    {
        $query = $this->selectQuery( $queryOptions );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );
        $rows = $result->fetchAll( PDO::FETCH_ASSOC );

        $aamaps = array();

        if ( $rows ) {
            foreach ( $rows as $row ) {
                $aamap = new OnpubAAMap();

                $aamap->setID( $row["ID"] );
                $aamap->setArticleID( $row["articleID"] );
                $aamap->setAuthorID( $row["authorID"] );
                $aamap->setCreated( new OnpubDateTime( $row["created"] ) );
                $aamap->setModified( new OnpubDateTime( $row["modified"] ) );

                $aamaps[] = $aamap;
            }
        }

        $result->closeCursor();
        return $aamaps;
    }

    private function selectQuery( OnpubQueryOptions $queryOptions )
    {
        $where = "";
        $orderBy = "";
        $limit = "";

        if ( $queryOptions->getDateLimit() ) {
            $where = "WHERE created <= " . $this->pdo->quote( $queryOptions->getDateLimit()->toString() );
        }

        if ( $queryOptions->getOrderBy() ) {
            $orderBy = "ORDER BY " . $queryOptions->getOrderBy();

            if ( $queryOptions->getOrder() ) {
                $orderBy .= " " . $queryOptions->getOrder();
            }
        }

        if ( $queryOptions->getPage() && $queryOptions->getRowLimit() && $queryOptions->getRowLimit() > 0 ) {
            $limit = "LIMIT " . (($queryOptions->getPage() - 1) * $queryOptions->getRowLimit()) . "," . $queryOptions->getRowLimit();
        }
        elseif ( $queryOptions->getRowLimit() && $queryOptions->getRowLimit() > 0 ) {
            $limit = "LIMIT 0," . $queryOptions->getRowLimit();
        }

        return "SELECT ID, articleID, authorID, created, modified FROM OnpubAAMaps $where $orderBy $limit";
    }

    /**
     * @param OnpubAAMap $aamap Map whose ID you want to get.
     * @return int The ID of the map. NULL if the map does not exist in the database.
     */
    public function getID( OnpubAAMap $aamap )
    {
        $stmt = $this->pdo->prepare( "SELECT ID FROM OnpubAAMaps WHERE articleID = :articleID AND authorID = :authorID" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );

        $stmt->bindParam( ':articleID', $aamap->getArticleID() );
        $stmt->bindParam( ':authorID', $aamap->getAuthorID() );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, false, $stmt->errorInfo() );

        if ( ($row = $stmt->fetch( PDO::FETCH_ASSOC )) ) {
            return $row["ID"];
        }

        return null;
    }

    /**
     * @param mixed $aamaps A single {@link OnpubAAMap} object or an array of {@link OnpubAAMap} objects (to insert multiple maps at a time).
     * @return mixed The ID(s) of the new map(s). An int will be returned if a single map was inserted. An array of ints will be returned if multiple maps were inserted.
     * @throws PDOException if there's a database error.
     */
    public function insert( $aamaps )
    {
        $aamapIDs = array();
        $isArray = true;

        if ( !is_array( $aamaps ) ) {
            $aamaps = array( $aamaps );
            $isArray = false;
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        $stmt = $this->pdo->prepare( "INSERT INTO OnpubAAMaps (ID, articleID, authorID, created, modified) VALUES (:ID, :articleID, :authorID, :created, :modified)" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        foreach ( $aamaps as $aamap ) {
            try {
                $aamapID = $this->getID( $aamap );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }

            if ( $aamapID ) {
                $aamapIDs[] = $aamapID;
                $aamap->setID( $aamapID );
            }
            else {
                $stmt->bindParam( ':ID', $aamap->getID() );
                $stmt->bindParam( ':articleID', $aamap->getArticleID() );
                $stmt->bindParam( ':authorID', $aamap->getAuthorID() );
                $stmt->bindParam( ':created', $aamap->getCreated()->toString() );
                $stmt->bindParam( ':modified', $aamap->getModified()->toString() );
                $result = $stmt->execute();
                OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

                $aamapIDs[] = $this->pdo->lastInsertId();
                $aamap->setID( $this->pdo->lastInsertId() );
            }
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        if ( $isArray ) {
            return $aamapIDs;
        }
        else {
            return end( $aamapIDs );
        }
    }
}

?>
