<?php

/**
 * Manage articles in the database.
 *
 * @author {@link mailto:corey@onpubco.com Corey HM Taylor}
 * @copyright Onpub (TM) Copyright (C) 2007, Onpubco
 * {@link http://onpubco.com/}
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * Version 2
 * @package onpubapi
 * @version 0.8.3
 */
class OnpubArticles
{
    private $pdo;
    /**
     * Controls whether or not the methods in this class use transactions when
     * they access the database.
     *
     * By default, this is set to true and database transactions will be used
     * where necessary. Set this to false if you want to manage transactions
     * yourself.
     * @var bool
     */
    public $enableTransactions;

    /**
     * Establish the database connection.
     *
     * @param PDO $pdo A {@link PHP_MANUAL#function.pdo-construct PHP Data Object}
     * @param bool $enableTransactions If true (the default), the methods in
     * this class that use database transactions will do so. If false, methods
     * that by default use transactions, will not. Only pass false to this
     * argument if you are managing transactions yourself.
     *
     */
    function __construct( PDO $pdo, $enableTransactions = true )
    {
        $this->pdo = $pdo;
        $this->enableTransactions = $enableTransactions;
        $this->pdo->setAttribute( PDO::ATTR_EMULATE_PREPARES, true );
    }

    /**
     * Returns the total number of articles in the database.
     *
     * @param int $sectionID Count articles mapped to given $sectionID.
     * @return int
     */
    public function count( $sectionID = null )
    {
        $query = $this->countQuery( $sectionID );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );

        if ( !($row = $result->fetch( PDO::FETCH_ASSOC )) ) {
            return 0;
        }

        $result->closeCursor();
        return $row["count"];
    }

    private function countQuery( $sectionID = null )
    {
        if ( $sectionID ) {
            return "SELECT COUNT(articles.ID) AS count FROM OnpubArticles AS articles LEFT JOIN OnpubSAMaps as samaps ON articles.ID = samaps.articleID WHERE sectionID = $sectionID";
        }
        else {
            return "SELECT COUNT(articles.ID) AS count FROM OnpubArticles AS articles";
        }
    }

    /**
     * Deletes an article from the database.
     *
     * @param int $ID ID of the article to delete.
     * @return int 1 if the article was deleted, 0 if the article does not exist in the database.
     */
    public function delete( $ID )
    {
        $oaamaps = new OnpubAAMaps( $this->pdo, false );
        $osamaps = new OnpubSAMaps( $this->pdo, false );

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        try {
            $oaamaps->delete( $ID, null );
        }
        catch ( PDOException $e ) {
            if ( $this->enableTransactions ) $this->pdo->rollBack();
            throw $e;
        }

        try {
            $osamaps->delete( null, $ID );
        }
        catch ( PDOException $e ) {
            if ( $this->enableTransactions ) $this->pdo->rollBack();
            throw $e;
        }

        $stmt = $this->pdo->prepare( "DELETE FROM OnpubArticles WHERE ID = :ID" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        $stmt->bindParam( ':ID', $ID );

        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        return $stmt->rowCount();
    }

    /**
     * Search for articles in the database.
     *
     * @param string $keywords Keyword(s) to search for.
     * @return array All the articles which were found as an array of {@link OnpubArticle} objects.
     */
    public function search( $keywords, OnpubQueryOptions $queryOptions )
    {
        $query = $this->findQuery( $keywords, $queryOptions );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );

        $rows = $result->fetchAll( PDO::FETCH_ASSOC );

        $articles = array();
        $authors = array();
        $lastAID = 0;
        $lastArticle = null;
        $content = "";

        if ( $rows ) {
            foreach ( $rows as $row ) {
                $currAID = $row["ID"];
                $currIID = $row["imageID"];
                $currAIID = $row["authorImageID"];

                if ( $lastAID != $currAID ) {
                    $authors = array();
                }

                $author = new OnpubAuthor();

                $author->setID( $row["authorID"] );
                $author->setImageID( $currAIID );
                $author->setGivenNames( stripslashes( $row["givenNames"] ) );
                $author->setFamilyName( stripslashes( $row["familyName"] ) );
                $author->setDisplayAs( stripslashes( $row["displayAs"] ) );
                $author->setCreated( new OnpubDateTime( $row["authorCreated"] ) );
                $author->setModified( new OnpubDateTime( $row["authorModified"] ) );

                $authors[] = $author;

                if ( $currAIID ) {
                    $s = sizeof( $authors );
                    $author = $authors[($s - 1)];

                    $image = new OnpubImage();

                    $image->setID( $currAIID );
                    $image->setWebsiteID( $row["authorImageWebsiteID"] );
                    $image->setFileName( stripslashes( $row["authorImageFileName"] ) );
                    $image->setDescription( stripslashes( $row["authorImageDescription"] ) );
                    $image->setCreated( new OnpubDateTime( $row["authorImageCreated"] ) );
                    $image->setModified( new OnpubDateTime( $row["authorImageModified"] ) );

                    $author->setImage( $image );
                }

                if ( $queryOptions->getIncludeContent() ) {
                    $content = stripslashes( $row["content"] );
                }
                else {
                    $content = "";
                }

                $currArticle = new OnpubArticle();

                $currArticle->setID( $currAID );
                $currArticle->setImageID( $currIID );
                $currArticle->setTitle( stripslashes( $row["title"] ) );
                $currArticle->setContent( $content );
                $currArticle->setURL( stripslashes( $row["url"] ) );
                $currArticle->setCreated( new OnpubDateTime( $row["created"] ) );
                $currArticle->setModified( new OnpubDateTime( $row["modified"] ) );

                if ( $currIID ) {
                    $image = new OnpubImage();

                    $image->setID( $currIID );
                    $image->setWebsiteID( $row["websiteID"] );
                    $image->setFileName( stripslashes( $row["fileName"] ) );
                    $image->setDescription( stripslashes( $row["description"] ) );
                    $image->setCreated( new OnpubDateTime( $row["imageCreated"] ) );
                    $image->setModified( new OnpubDateTime( $row["imageModified"] ) );

                    $currArticle->setImage( $image );
                }

                if ( $lastArticle != null ) {
                    if ( $lastArticle->getID() == $currArticle->getID() ) {
                        $lastArticle->setAuthors( $authors );
                    }
                    else {
                        $articles[] = $currArticle;
                        $currArticle->setAuthors( $authors );
                    }
                }
                else {
                    $articles[] = $currArticle;
                    $currArticle->setAuthors( $authors );
                }

                $lastAID = $currAID;

                if ( $lastArticle != null ) {
                    if ( $lastArticle->getID() != $currArticle->getID() ) {
                        $lastArticle = $currArticle;
                    }
                }
                else {
                    $lastArticle = $currArticle;
                }
            }
        }

        $result->closeCursor();
        return $articles;
    }

    private function findQuery( $keywords, $queryOptions )
    {
        $keywords = OnpubDatabase::utf8Decode( trim( $keywords ) );
        $where = "";

        if ( $keywords ) {
            if ( $queryOptions->getFullTextSearch() ) {
                $where = "WHERE articles.title LIKE '%$keywords%' OR authors.displayAS LIKE '%$keywords%' OR articles.created LIKE '%$keywords%' OR articles.modified LIKE '%$keywords%' OR articles.ID LIKE '%$keywords%' OR articles.content LIKE '%$keywords%' OR articles.url LIKE '%$keywords%'";
            }
            else {
                $where = "WHERE articles.title LIKE '%$keywords%' OR authors.displayAS LIKE '%$keywords%' OR articles.created LIKE '%$keywords%' OR articles.modified LIKE '%$keywords%' OR articles.ID LIKE '%$keywords%' OR articles.url LIKE '%$keywords%'";
            }
        }

        if ( $queryOptions->getOrderBy() ) {
            $orderBy = $queryOptions->getOrderBy();
        }
        else {
            $orderBy = "created";
        }

        if ( $queryOptions->getOrder() ) {
            $order = $queryOptions->getOrder();
        }
        else {
            $order = "DESC";
        }

        if ( $queryOptions->getIncludeContent() ) {
            $articleColumns = "articles.ID, articles.imageID, title, content, articles.url, articles.created, articles.modified";
        }
        else {
            $articleColumns = "articles.ID, articles.imageID, title, articles.url,  articles.created, articles.modified";
        }

        if ( $queryOptions->getDateLimit() ) {
            $dateLimit = $queryOptions->getDateLimit()->toString();

            if ( $keywords ) {
                $where .= " AND articles.created <= '$dateLimit'";
            }
            else {
                $where = "WHERE articles.created <= '$dateLimit'";
            }
        }

        return "SELECT $articleColumns, articleimages.websiteID, articleimages.fileName, articleimages.description, articleimages.created AS imageCreated, articleimages.modified AS imageModified, authors.ID AS authorID, authors.imageID AS authorImageID, givenNames, familyName, displayAs, authors.created AS authorCreated, authors.modified AS authorModified, authorimages.websiteID AS authorImageWebsiteID, authorimages.fileName AS authorImageFileName, authorimages.description AS authorImageDescription, authorimages.created AS authorImageCreated, authorimages.modified AS authorImageModified FROM OnpubArticles AS articles LEFT JOIN OnpubImages AS articleimages ON articles.imageID = articleimages.ID LEFT JOIN OnpubAAMaps AS aamaps ON articles.ID = aamaps.articleID LEFT JOIN OnpubAuthors AS authors ON aamaps.authorID = authors.ID LEFT JOIN OnpubImages AS authorimages ON authors.imageID = authorimages.ID $where ORDER BY articles.$orderBy $order";
    }

    /**
     * Get a single article from the database.
     *
     * @param int $ID ID of the article to get.
     * @param OnpubQueryOptions $queryOptions Optional database query options.
     * @return OnpubArticle An {@link OnpubArticle} object. NULL if the article does not exist in the database.
     */
    public function get( $ID, $queryOptions = null )
    {
        if ( !$queryOptions ) $queryOptions = new OnpubQueryOptions();

        $query = $this->getQuery( $ID, $queryOptions );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );
        $rows = $result->fetchAll( PDO::FETCH_ASSOC );

        if ( !$rows ) {
            return null;
        }

        $row = $rows[0];

        if ( $queryOptions->getIncludeContent() ) {
            $content = stripslashes( $row["content"] );
        }
        else {
            $content = "";
        }

        $article = new OnpubArticle();

        $article->setID( $row["ID"] );
        $article->setImageID( $row["imageID"] );
        $article->setTitle( stripslashes( $row["title"] ) );
        $article->setContent( $content );
        $article->setURL( stripslashes( $row["url"] ) );
        $article->setCreated( new OnpubDateTime( $row["created"] ) );
        $article->setModified( new OnpubDateTime( $row["modified"] ) );

        if ( !$queryOptions->getIncludeAuthors() ) {
            return $article;
        }

        if ( $row["imageID"] ) {
            $image = new OnpubImage();

            $image->setID( $row["imageID"] );
            $image->setWebsiteID( $row["websiteID"] );
            $image->setFileName( stripslashes( $row["fileName"] ) );
            $image->setDescription( stripslashes( $row["description"] ) );
            $image->setCreated( new OnpubDateTime( $row["imageCreated"] ) );
            $image->setModified( new OnpubDateTime( $row["imageModified"] ) );

            $article->setImage( $image );
        }

        $authorsassoc = array();

        foreach ( $rows as $row ) {
            $authorID = $row["authorID"];

            if ( $authorID ) {
                $author = new OnpubAuthor();

                $author->setID( $authorID );
                $author->setImageID( $row["authorImageID"] );
                $author->setGivenNames( stripslashes( $row["givenNames"] ) );
                $author->setFamilyName( stripslashes( $row["familyName"] ) );
                $author->setDisplayAs( stripslashes( $row["displayAs"] ) );
                $author->setCreated( new OnpubDateTime( $row["authorCreated"] ) );
                $author->setModified( new OnpubDateTime( $row["authorModified"] ) );

                if ( isset( $authorsassoc["$authorID"] ) ) {
                    $author = $authorsassoc["$authorID"];
                }
                else {
                    $authorsassoc["$authorID"] = $author;
                }

                if ( $row["authorImageID"] ) {
                    $image = new OnpubImage();

                    $image->setID( $row["authorImageID"] );
                    $image->setWebsiteID( $row["authorImageWebsiteID"] );
                    $image->setFileName( stripslashes( $row["authorImageFileName"] ) );
                    $image->setDescription( stripslashes( $row["authorImageDescription"] ) );
                    $image->setCreated( new OnpubDateTime( $row["authorImageCreated"] ) );
                    $image->setModified( new OnpubDateTime( $row["authorImageModified"] ) );

                    $author->setImage( $image );
                }

                $authors = $article->getAuthors();
                $exists = false;

                foreach ( $authors as $a ) {
                    if ( $a->getID() == $author->getID() ) {
                        $exists = true;
                    }
                }

                if ( $exists == false ) {
                    $authors[] = $author;
                    $article->setAuthors( $authors );
                }
            }
        }

        $result->closeCursor();
        return $article;
    }

    private function getQuery( $ID, $queryOptions = null )
    {
        if ( !$queryOptions ) $queryOptions = new OnpubQueryOptions();

        if ( $queryOptions->getOrderBy() ) {
            $orderBy = $queryOptions->getOrderBy();
            $orderBy = "articles.$orderBy";
        }
        else {
            $orderBy = "aamaps.ID";
        }

        if ( $queryOptions->getOrder() ) {
            $order = $queryOptions->getOrder();
        }
        else {
            $order = "";
        }

        $where = "WHERE articles.ID = $ID";

        if ( $queryOptions->getDateLimit() ) {
            $dateLimit = $queryOptions->getDateLimit()->toString();
            $where .= " AND articles.created <= '$dateLimit'";
        }

        if ( $queryOptions->getIncludeContent() ) {
            $articleColumns = "articles.ID, articles.imageID, title, content, articles.url, articles.created, articles.modified";
        }
        else {
            $articleColumns = "articles.ID, articles.imageID, title, articles.url, articles.created, articles.modified";
        }

        if ( $queryOptions->getIncludeAuthors() ) {
            return "SELECT $articleColumns, articleimages.websiteID, articleimages.fileName, articleimages.description, articleimages.created AS imageCreated, articleimages.modified AS imageModified, authors.ID AS authorID, authors.imageID AS authorImageID, givenNames, familyName, displayAs, authors.created AS authorCreated, authors.modified AS authorModified, authorimages.websiteID AS authorImageWebsiteID, authorimages.fileName AS authorImageFileName, authorimages.description AS authorImageDescription, authorimages.created AS authorImageCreated, authorimages.modified AS authorImageModified FROM OnpubArticles AS articles LEFT JOIN OnpubImages AS articleimages ON articles.imageID = articleimages.ID LEFT JOIN OnpubAAMaps AS aamaps ON articles.ID = aamaps.articleID LEFT JOIN OnpubAuthors AS authors ON aamaps.authorID = authors.ID LEFT JOIN OnpubImages AS authorimages ON authors.imageID = authorimages.ID $where ORDER BY $orderBy $order";
        }
        else {
            return "SELECT $articleColumns FROM OnpubArticles as articles WHERE ID = $ID";
        }
    }

    /**
     * Get the ID of an article in the database.
     *
     * Looks for an article in the database with the same title and content.
     * If there's a match, the ID of the article is returned.
     *
     * @param OnpubArticle $article Article whose ID you want to get.
     * @return int The ID of the article. NULL if the article does not exist in the database.
     */
    public function getID( OnpubArticle $article )
    {
        $stmt = $this->pdo->prepare( "SELECT ID FROM OnpubArticles WHERE title = :title AND content = :content" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );

        $stmt->bindParam( ':title', OnpubDatabase::utf8Decode( trim( $article->getTitle() ) ) );
        $stmt->bindParam( ':content', OnpubDatabase::utf8Decode( trim( $article->getContent() ) ) );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, false, $stmt->errorInfo() );

        if ( ($row = $stmt->fetch( PDO::FETCH_ASSOC )) ) {
            return $row["ID"];
        }

        return null;
    }

    /**
     * Insert new articles into the database.
     *
     * @param mixed $articles A single {@link OnpubArticle} object or an array of {@link OnpubArticle} objects (to insert multiple articles at a time).
     * @return mixed The ID(s) of the new article(s). An int will be returned if a single article was inserted. An array of ints will be returned if multiple articles were inserted.
     * @throws PDOException if there's a database error.
     */
    public function insert( $articles )
    {
        $oimages = new OnpubImages( $this->pdo, false );
        $oauthors = new OnpubAuthors( $this->pdo, false );
        $oaamaps = new OnpubAAMaps( $this->pdo, false );
        $osamaps = new OnpubSAMaps( $this->pdo, false );
        $IDs = array();
        $isArray = true;

        if ( !is_array( $articles ) ) {
            $articles = array( $articles );
            $isArray = false;
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        $stmt = $this->pdo->prepare( "INSERT INTO OnpubArticles (ID, imageID, title, content, url, created, modified) VALUES (:ID, :imageID, :title, :content, :url, :created, :modified)" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        foreach ( $articles as $article ) {
            if ( $article->getImage() ) {
                try {
                    $imageID = $oimages->insert( $article->getImage() );
                    $article->setImageID( $imageID );
                }
                catch ( PDOException $e ) {
                    if ( $this->enableTransactions ) $this->pdo->rollBack();
                    throw $e;
                }
            }

            try {
                $ID = $this->getID( $article );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }

            if ( $ID ) {
                $IDs[] = $ID;
                $article->setID( $ID );
            }
            else {
                $stmt->bindParam( ':ID', $article->getID() );
                $stmt->bindParam( ':imageID', $article->getImageID() );
                $stmt->bindParam( ':title', OnpubDatabase::utf8Decode( trim( $article->getTitle() ) ) );
                $stmt->bindParam( ':content', OnpubDatabase::utf8Decode( trim( $article->getContent() ) ) );
                $stmt->bindParam( ':url', OnpubDatabase::utf8Decode( trim( $article->getURL() ) ) );
                $stmt->bindParam( ':created', $article->getCreated()->toString() );
                $stmt->bindParam( ':modified', $article->getModified()->toString() );
                $result = $stmt->execute();
                OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

                $IDs[] = $this->pdo->lastInsertId();
                $article->setID( $this->pdo->lastInsertId() );
            }

            $authors = $article->getAuthors();

            try {
                $oauthors->insert( $authors );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }

            $aamaps = array();

            foreach ( $authors as $author ) {
                $aamap = new OnpubAAMap();

                $aamap->setArticleID( $article->getID() );
                $aamap->setAuthorID( $author->getID() );
                $aamap->setCreated( $article->getCreated() );
                $aamap->setModified( $article->getModified() );

                $aamaps[] = $aamap;
            }

            try {
                $oaamaps->insert( $aamaps );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }

            $sectionIDs = $article->getSectionIDs();
            $samaps = array();

            foreach ( $sectionIDs as $sectionID ) {
                $samap = new OnpubSAMap();

                $samap->setSectionID( $sectionID );
                $samap->setArticleID( $article->getID() );
                $samap->setCreated( $article->getCreated() );
                $samap->setModified( $article->getModified() );

                $samaps[] = $samap;
            }

            try {
                $osamaps->insert( $samaps );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        if ( $isArray ) {
            return $IDs;
        }
        else {
            return end( $IDs );
        }
    }

    /**
     * Get mulitple articles from the database.
     *
     * @param OnpubQueryOptions $queryOptions Database query options.
     * @return array An array of {@link OnpubArticle} objects.
     */
    public function select( OnpubQueryOptions $queryOptions, $sectionID = null, $websiteID = null )
    {
        $query = $this->selectQuery( $queryOptions, $sectionID, $websiteID );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );
        $rows = $result->fetchAll( PDO::FETCH_ASSOC );

        $articles = array();

        if ( $rows ) {
            foreach ( $rows as $row ) {
                $article = new OnpubArticle();

                if ( $queryOptions->getIncludeContent() ) {
                    $content = stripslashes( $row["content"] );

                }
                else {
                    $content = "";
                }

                $article->setID( $row["ID"] );
                $article->setImageID( $row["imageID"] );
                $article->setTitle( stripslashes( $row["title"] ) );
                $article->setContent( $content );
                $article->setURL( stripslashes( $row["url"] ) );
                $article->setCreated( new OnpubDateTime( $row["created"] ) );
                $article->setModified( new OnpubDateTime( $row["modified"] ) );

                if ( $sectionID || $websiteID ) {
                    $article->appendSectionID( $row["sectionID"] );
                }

                $articles[] = $article;
            }
        }

        $result->closeCursor();
        return $articles;
    }

    private function selectQuery( $queryOptions, $sectionID = null, $websiteID = null )
    {
        $articleColumns = "articles.ID, articles.imageID, articles.title, articles.url, articles.created, articles.modified";
        $where = "";
        $orderBy = "";
        $limit = "";

        if ( $queryOptions->getIncludeContent() ) {
            $articleColumns = "articles.ID, articles.imageID, articles.title, articles.content, articles.url, articles.created, articles.modified";
        }

        if ( $sectionID || $websiteID ) {
            $articleColumns .= ", samaps.sectionID AS sectionID";
        }

        if ( $queryOptions->getDateLimit() ) {
            $where = "WHERE articles.created <= " . $this->pdo->quote( $queryOptions->getDateLimit()->toString() );

            if ( $sectionID && !$websiteID ) {
                $where .= " AND samaps.sectionID = $sectionID";
            }

            if ( $websiteID && !$sectionID ) {
                $where .= " AND wsmaps.websiteID = $websiteID";
            }
        }
        else {
            if ( $sectionID && !$websiteID ) {
                $where = "WHERE samaps.sectionID = $sectionID";
            }

            if ( $websiteID && !$sectionID ) {
                $where = "WHERE wsmaps.websiteID = $websiteID";
            }
        }

        if ( $queryOptions->getOrderBy() ) {
            $orderBy = "ORDER BY articles." . $queryOptions->getOrderBy();

            if ( $queryOptions->getOrder() ) {
                $orderBy .= " " . $queryOptions->getOrder();
            }
        }

        if ( $queryOptions->getPage() && $queryOptions->getRowLimit() && $queryOptions->getRowLimit() > 0 ) {
            $limit = "LIMIT " . (($queryOptions->getPage() - 1) * $queryOptions->getRowLimit()) . "," . $queryOptions->getRowLimit();
        }
        elseif ( $queryOptions->getRowLimit() && $queryOptions->getRowLimit() > 0 ) {
            $limit = "LIMIT 0," . $queryOptions->getRowLimit();
        }

        if ( $sectionID && !$websiteID ) {
            return "SELECT $articleColumns FROM OnpubArticles AS articles LEFT JOIN OnpubSAMaps AS samaps ON articles.ID = samaps.articleID $where $orderBy $limit";
        }
        elseif ( $websiteID && !$sectionID ) {
            return "SELECT $articleColumns FROM OnpubArticles AS articles LEFT JOIN OnpubSAMaps AS samaps ON articles.ID = samaps.articleID LEFT JOIN OnpubWSMaps AS wsmaps ON samaps.sectionID = wsmaps.sectionID $where $orderBy $limit";
        }
        else {
            return "SELECT $articleColumns FROM OnpubArticles AS articles $where $orderBy $limit";
        }
    }

    /**
     * Update an existing article in the database.
     *
     * If you set the article's sectionIDs to NULL, it will be unmapped from
     * any sections it was previously mapped to.
     *
     * @param OnpubArticle $article The article to be updated.
     * @param bool $overwriteAuthorMaps False by default. If set to true, the
     * author mappings for this article will be deleted and recreated, if
     * applicable.
     * @return int 1 if the article was updated. 0 if the article does not exist in the database.
     */
    public function update( OnpubArticle $article, $overwriteAuthorMaps = false )
    {
        $oaamaps = new OnpubAAMaps( $this->pdo, false );
        $oauthors = new OnpubAuthors( $this->pdo, false );
        $osamaps = new OnpubSAMaps( $this->pdo, false );
        $oimages = new OnpubImages( $this->pdo, false );
        $now = new OnpubDateTime();

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        $stmt = $this->pdo->prepare( "UPDATE OnpubArticles SET imageID = :imageID, title = :title, content = :content, url = :url, created = :created, modified = :modified WHERE ID = :ID" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        if ( $article->getImage() ) {
            try {
                $imageID = $oimages->insert( $article->getImage() );
                $article->setImageID( $imageID );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }
        }

        $stmt->bindParam( ':imageID', $article->getImageID() );
        $stmt->bindParam( ':title', OnpubDatabase::utf8Decode( trim( $article->getTitle() ) ) );
        $stmt->bindParam( ':content', OnpubDatabase::utf8Decode( trim( $article->getContent() ) ) );
        $stmt->bindParam( ':url', OnpubDatabase::utf8Decode( trim( $article->getURL() ) ) );
        $stmt->bindParam( ':created', $article->getCreated()->toString() );
        $stmt->bindParam( ':modified', $now->toString() );
        $stmt->bindParam( ':ID', $article->getID() );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

        if ( $overwriteAuthorMaps ) {
            try {
                $oaamaps->delete( $article->getID(), null );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }
        }

        $authors = $article->getAuthors();

        foreach ( $authors as $author ) {
            if ( $author->getID() ) {
                try {
                    $oauthors->update( $author );
                }
                catch ( PDOException $e ) {
                    if ( $this->enableTransactions ) $this->pdo->rollBack();
                    throw $e;
                }
            }
            else {
                try {
                    $oauthors->insert( $author );
                }
                catch ( PDOException $e ) {
                    if ( $this->enableTransactions ) $this->pdo->rollBack();
                    throw $e;
                }
            }

            try {
                $aamap = new OnpubAAMap();
                $aamap->setArticleID( $article->getID() );
                $aamap->setAuthorID( $author->getID() );
                $oaamaps->insert( $aamap );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }
        }

        $sectionIDs = $article->getSectionIDs();

        if ( $sectionIDs === null ) {
            try {
                $samaps = $osamaps->delete( null, $article->getID() );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }
        }
        elseif ( sizeof( $sectionIDs ) ) {
            $queryOptions = new OnpubQueryOptions();
            $queryOptions->setOrderBy( "ID" );
            $queryOptions->setOrder( "ASC" );

            try {
                $samaps = $osamaps->select( $queryOptions, null, $article->getID() );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }

            foreach ( $samaps as $samap ) {
                if ( !in_array( $samap->getSectionID(), $sectionIDs ) ) {
                    try {
                        $osamaps->delete( $samap->getSectionID(), $article->getID() );
                    }
                    catch ( PDOException $e ) {
                        if ( $this->enableTransactions ) $this->pdo->rollBack();
                        throw $e;
                    }
                }
            }

            foreach ( $sectionIDs as $sectionID ) {
                $samap = new OnpubSAMap();

                $samap->setSectionID( $sectionID );
                $samap->setArticleID( $article->getID() );
                $samap->setCreated( $article->getCreated() );
                $samap->setModified( $article->getModified() );

                try {
                    $samapID = $osamaps->getID( $samap );
                }
                catch ( PDOException $e ) {
                    if ( $this->enableTransactions ) $this->pdo->rollBack();
                    throw $e;
                }

                if ( $samapID ) {
                    $samap->setID( $samapID );

                    try {
                        $osamaps->update( $samap );
                    }
                    catch ( PDOException $e ) {
                        if ( $this->enableTransactions ) $this->pdo->rollBack();
                        throw $e;
                    }
                }
                else {
                    try {
                        $osamaps->insert( $samap );
                    }
                    catch ( PDOException $e ) {
                        if ( $this->enableTransactions ) $this->pdo->rollBack();
                        throw $e;
                    }
                }
            }
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        return $stmt->rowCount();
    }
}

?>
