<?php

/**
 * @author {@link mailto:corey@onpubco.com Corey HM Taylor}
 * @copyright Onpub (TM) Copyright (C) 2007, Onpubco
 * {@link http://onpubco.com/}
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * Version 2
 * @package onpubapi
 * @version 0.8.3
 */
class OnpubAuthors
{
    private $pdo;
    /**
     * @var bool
     */
    public $enableTransactions;

    /**
     * @param PDO $pdo A {@link PHP_MANUAL#function.pdo-construct PHP Data Object}
     * @param bool $enableTransactions
     */
    function __construct( PDO $pdo, $enableTransactions = true )
    {
        $this->pdo = $pdo;
        $this->enableTransactions = $enableTransactions;
        $this->pdo->setAttribute( PDO::ATTR_EMULATE_PREPARES, true );
    }

    /**
     * @return int
     */
    public function count()
    {
        $query = $this->countQuery();
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );

        if ( !($row = $result->fetch( PDO::FETCH_ASSOC )) ) {
            return 0;
        }

        $result->closeCursor();
        return $row["count"];
    }

    private function countQuery()
    {
        return "SELECT COUNT(ID) AS count FROM OnpubAuthors";
    }

    /**
     * @param int $ID ID of the author to get.
     * @return OnpubAuthor An {@link OnpubAuthor} object. NULL if the author does not exist in the database.
     */
    public function get( $ID )
    {
        $query = $this->getQuery( $ID );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );

        if ( !($row = $result->fetch( PDO::FETCH_ASSOC )) ) {
            return null;
        }

        $author = new OnpubAuthor();

        $author->setID( $row["ID"] );
        $author->setImageID( $row["imageID"] );
        $author->setGivenNames( stripslashes( $row["givenNames"] ) );
        $author->setFamilyName( stripslashes( $row["familyName"] ) );
        $author->setDisplayAs( stripslashes( $row["displayAs"] ) );
        $author->setCreated( new OnpubDateTime( $row["created"] ) );
        $author->setModified( new OnpubDateTime( $row["modified"] ) );

        $result->closeCursor();
        return $author;
    }

    private function getQuery( $ID )
    {
        return "SELECT * FROM OnpubAuthors WHERE ID = $ID";
    }

    /**
     * @param OnpubAuthor $author Author whose ID you want to get.
     * @return int The ID of the author. NULL if the author does not exist in the database.
     */
    public function getID( OnpubAuthor $author )
    {
        $stmt = $this->pdo->prepare( "SELECT ID FROM OnpubAuthors WHERE displayAs = :displayAs" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );

        $stmt->bindParam( ':displayAs', OnpubDatabase::utf8Decode( trim( $author->getDisplayAs() ) ) );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, false, $stmt->errorInfo() );

        if ( ($row = $stmt->fetch( PDO::FETCH_ASSOC )) ) {
            return $row["ID"];
        }

        return null;
    }

    /**
     * @param OnpubQueryOptions $queryOptions Database query options.
     * @return array An array of {@link OnpubAuthor} objects.
     */
    public function select( OnpubQueryOptions $queryOptions )
    {
        $query = $this->selectQuery( $queryOptions );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );
        $rows = $result->fetchAll( PDO::FETCH_ASSOC );

        $authors = array();

        if ( $rows ) {
            foreach ( $rows as $row ) {
                $author = new OnpubAuthor();

                $author->setID( $row["ID"] );
                $author->setImageID( $row["imageID"] );
                $author->setGivenNames( stripslashes( $row["givenNames"] ) );
                $author->setFamilyName( stripslashes( $row["familyName"] ) );
                $author->setDisplayAs( stripslashes( $row["displayAs"] ) );
                $author->setCreated( new OnpubDateTime( $row["created"] ) );
                $author->setModified( new OnpubDateTime( $row["modified"] ) );

                $authors[] = $author;
            }
        }

        $result->closeCursor();
        return $authors;
    }

    private function selectQuery( OnpubQueryOptions $queryOptions )
    {
        $where = "";
        $orderBy = "";
        $limit = "";

        if ( $queryOptions->getDateLimit() ) {
            $where = "WHERE created <= " . $this->pdo->quote( $queryOptions->getDateLimit()->toString() );
        }

        if ( $queryOptions->getOrderBy() ) {
            $orderBy = "ORDER BY " . $queryOptions->getOrderBy();

            if ( $queryOptions->getOrder() ) {
                $orderBy .= " " . $queryOptions->getOrder();
            }
        }

        if ( $queryOptions->getPage() && $queryOptions->getRowLimit() && $queryOptions->getRowLimit() > 0 ) {
            $limit = "LIMIT " . (($queryOptions->getPage() - 1) * $queryOptions->getRowLimit()) . "," . $queryOptions->getRowLimit();
        }
        elseif ( $queryOptions->getRowLimit() && $queryOptions->getRowLimit() > 0 ) {
            $limit = "LIMIT 0," . $queryOptions->getRowLimit();
        }

        return "SELECT ID, imageID, givenNames, familyName, displayAs, created, modified FROM OnpubAuthors $where $orderBy $limit";
    }

    /**
     * @param mixed $authors A single {@link OnpubAuthor} object or an array of {@link OnpubAuthor} objects (to insert multiple authors at a time).
     * @return mixed The ID(s) of the new author(s). An int will be returned if a single author was inserted. An array of ints will be returned if multiple authors were inserted.
     * @throws PDOException if there's a database error.
     */
    public function insert( $authors )
    {
        $oimages = new OnpubImages( $this->pdo, false );
        $IDs = array();
        $isArray = true;

        if ( !is_array( $authors ) ) {
            $authors = array( $authors );
            $isArray = false;
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        $stmt = $this->pdo->prepare( "INSERT INTO OnpubAuthors (ID, imageID, givenNames, familyName, displayAs, created, modified) VALUES (:ID, :imageID, :givenNames, :familyName, :displayAs, :created, :modified)" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        foreach ( $authors as $author ) {
            if ( $author->getImage() ) {
                try {
                    $imageID = $oimages->insert( $author->getImage() );
                    $author->setImageID( $imageID );
                }
                catch ( PDOException $e ) {
                    if ( $this->enableTransactions ) $this->pdo->rollBack();
                    throw $e;
                }
            }

            try {
                $ID = $this->getID( $author );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }

            if ( $ID ) {
                $IDs[] = $ID;
                $author->setID( $ID );
            }
            else {
                $stmt->bindParam( ':ID', $author->getID() );
                $stmt->bindParam( ':imageID', $author->getImageID() );
                $stmt->bindParam( ':givenNames', OnpubDatabase::utf8Decode( trim( $author->getGivenNames() ) ) );
                $stmt->bindParam( ':familyName', OnpubDatabase::utf8Decode( trim( $author->getFamilyName() ) ) );
                $stmt->bindParam( ':displayAs', OnpubDatabase::utf8Decode( trim( $author->getDisplayAs() ) ) );
                $stmt->bindParam( ':created', $author->getCreated()->toString() );
                $stmt->bindParam( ':modified', $author->getModified()->toString() );
                $result = $stmt->execute();
                OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

                $IDs[] = $this->pdo->lastInsertId();
                $author->setID( $this->pdo->lastInsertId() );
            }
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        if ( $isArray ) {
            return $IDs;
        }
        else {
            return end( $IDs );
        }
    }

    /**
     * @param OnpubAuthor $author The author to be updated.
     * @return int 1 if the author was updated. 0 if the author does not exist in the database.
     */
    public function update( OnpubAuthor $author )
    {
        $now = new OnpubDateTime();

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        $stmt = $this->pdo->prepare( "UPDATE OnpubAuthors SET imageID = :imageID, givenNames = :givenNames, familyName = :familyName, displayAs = :displayAs, modified = :modified WHERE ID = :ID" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        $stmt->bindParam( ':imageID', $author->getImageID() );
        $stmt->bindParam( ':givenNames', OnpubDatabase::utf8Decode( trim( $author->getGivenNames() ) ) );
        $stmt->bindParam( ':familyName', OnpubDatabase::utf8Decode( trim( $author->getFamilyName() ) ) );
        $stmt->bindParam( ':displayAs', OnpubDatabase::utf8Decode( trim( $author->getDisplayAs() ) ) );
        $stmt->bindParam( ':modified', $now->toString() );
        $stmt->bindParam( ':ID', $author->getID() );

        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        return $stmt->rowCount();
    }
}

?>
