<?php

/**
 * @author {@link mailto:corey@onpubco.com Corey HM Taylor}
 * @copyright Onpub (TM) Copyright (C) 2007, Onpubco
 * {@link http://onpubco.com/}
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * Version 2
 * @package onpubapi
 * @version 0.8.3
 */
class OnpubImages
{
    private $pdo;
    /**
     * @var bool
     */
    public $enableTransactions;

    /**
     * @param PDO $pdo A {@link PHP_MANUAL#function.pdo-construct PHP Data Object}
     * @param bool $enableTransactions
     */
    function __construct( PDO $pdo, $enableTransactions = true )
    {
        $this->pdo = $pdo;
        $this->enableTransactions = $enableTransactions;
        $this->pdo->setAttribute( PDO::ATTR_EMULATE_PREPARES, true );
    }

    /**
     * @return int
     */
    public function count()
    {
        $query = $this->countQuery();
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );

        if ( !($row = $result->fetch( PDO::FETCH_ASSOC )) ) {
            return 0;
        }

        $result->closeCursor();
        return $row["count"];
    }

    private function countQuery()
    {
        return "SELECT COUNT(ID) AS count FROM OnpubImages";
    }

    /**
     * @param mixed $images A single {@link OnpubImage} object or an array of {@link OnpubImage} objects (to insert multiple images at a time).
     * @return mixed The ID(s) of the new image(s). An int will be returned if a single image was inserted. An array of ints will be returned if multiple images were inserted.
     * @throws PDOException if there's a database error.
     */
    public function insert( $images )
    {
        $IDs = array();
        $isArray = true;

        if ( !is_array( $images ) ) {
            $images = array( $images );
            $isArray = false;
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        $stmt = $this->pdo->prepare( "INSERT INTO OnpubImages (ID, websiteID, fileName, description, created, modified) VALUES (:ID, :websiteID, :fileName, :description, :created, :modified)" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        foreach ( $images as $image ) {
            try {
                $ID = $this->getID( $image );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }

            if ( $ID ) {
                $IDs[] = $ID;
                $image->setID( $ID );
            }
            else {
                $stmt->bindParam( ':ID', $image->getID() );
                $stmt->bindParam( ':websiteID', $image->getWebsiteID() );
                $stmt->bindParam( ':fileName', OnpubDatabase::utf8Decode( trim( $image->getFileName() ) ) );
                $stmt->bindParam( ':description', OnpubDatabase::utf8Decode( trim( $image->getDescription() ) ) );
                $stmt->bindParam( ':created', $image->getCreated()->toString() );
                $stmt->bindParam( ':modified', $image->getModified()->toString() );
                $result = $stmt->execute();
                OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

                $IDs[] = $this->pdo->lastInsertId();
                $image->setID( $this->pdo->lastInsertId() );
            }
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        if ( $isArray ) {
            return $IDs;
        }
        else {
            return end( $IDs );
        }
    }

    /**
     * @param int $ID ID of the image to get.
     * @return OnpubImage An {@link OnpubImage} object. NULL if the image does not exist in the database.
     */
    public function get( $ID )
    {
        $query = $this->getQuery( $ID );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );

        if ( !($row = $result->fetch( PDO::FETCH_ASSOC )) ) {
            return null;
        }

        $image = new OnpubImage();

        $image->setID( $row["ID"] );
        $image->setWebsiteID( $row["websiteID"] );
        $image->setFileName( stripslashes( $row["fileName"] ) );
        $image->setDescription( stripslashes( $row["description"] ) );
        $image->setCreated( new OnpubDateTime( $row["created"] ) );
        $image->setModified( new OnpubDateTime( $row["modified"] ) );

        $result->closeCursor();
        return $image;
    }

    private function getQuery( $ID )
    {
        return "SELECT * FROM OnpubImages WHERE ID = $ID";
    }

    /**
     * @param OnpubImage $image Image whose ID you want to get.
     * @return int The ID of the image. NULL if the image does not exist in the database.
     */
    public function getID( OnpubImage $image )
    {
        $stmt = $this->pdo->prepare( "SELECT ID FROM OnpubImages WHERE websiteID = :websiteID AND fileName = :fileName" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );

        $stmt->bindParam( ':websiteID', $image->getWebsiteID() );
        $stmt->bindParam( ':fileName', OnpubDatabase::utf8Decode( trim( $image->getFileName() ) ) );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, false, $stmt->errorInfo() );

        if ( ($row = $stmt->fetch( PDO::FETCH_ASSOC )) ) {
            return $row["ID"];
        }

        return null;
    }

    /**
     * @param OnpubQueryOptions $queryOptions Database query options.
     * @return array An array of {@link OnpubImage} objects.
     */
    public function select( OnpubQueryOptions $queryOptions )
    {
        $query = $this->selectQuery( $queryOptions );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );
        $rows = $result->fetchAll( PDO::FETCH_ASSOC );

        $images = array();

        if ( $rows ) {
            foreach ( $rows as $row ) {
                $image = new OnpubImage();

                $image->setID( $row["ID"] );
                $image->setWebsiteID( $row["websiteID"] );
                $image->setFileName( stripslashes( $row["fileName"] ) );
                $image->setDescription( stripslashes( $row["description"] ) );
                $image->setCreated( new OnpubDateTime( $row["created"] ) );
                $image->setModified( new OnpubDateTime( $row["modified"] ) );

                $images[] = $image;
            }
        }

        $result->closeCursor();
        return $images;
    }

    private function selectQuery( OnpubQueryOptions $queryOptions )
    {
        $where = "";
        $orderBy = "";
        $limit = "";

        if ( $queryOptions->getDateLimit() ) {
            $where = "WHERE created <= " . $this->pdo->quote( $queryOptions->getDateLimit()->toString() );
        }

        if ( $queryOptions->getOrderBy() ) {
            $orderBy = "ORDER BY " . $queryOptions->getOrderBy();

            if ( $queryOptions->getOrder() ) {
                $orderBy .= " " . $queryOptions->getOrder();
            }
        }

        if ( $queryOptions->getPage() && $queryOptions->getRowLimit() && $queryOptions->getRowLimit() > 0 ) {
            $limit = "LIMIT " . (($queryOptions->getPage() - 1) * $queryOptions->getRowLimit()) . "," . $queryOptions->getRowLimit();
        }
        elseif ( $queryOptions->getRowLimit() && $queryOptions->getRowLimit() > 0 ) {
            $limit = "LIMIT 0," . $queryOptions->getRowLimit();
        }

        return "SELECT ID, websiteID, fileName, description, created, modified FROM OnpubImages $where $orderBy $limit";
    }

    /**
     * @param string $keywords Keyword(s) to search for.
     * @return array All the images which were found as an array of {@link OnpubImage} objects.
     */
    public function search( $keywords, $orderBy = null, $order = null )
    {
        $query = $this->findQuery( $keywords, $orderBy, $order );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );

        $rows = $result->fetchAll( PDO::FETCH_ASSOC );

        $images = array();

        if ( $rows ) {
            foreach ( $rows as $row ) {
                $image = new OnpubImage();

                $image->setID( $row["ID"] );
                $image->setWebsiteID( $row["websiteID"] );
                $image->setFileName( stripslashes( $row["fileName"] ) );
                $image->setDescription( stripslashes( $row["description"] ) );
                $image->setCreated( new OnpubDateTime( $row["created"] ) );
                $image->setModified( new OnpubDateTime( $row["modified"] ) );

                $images[] = $image;
            }
        }

        $result->closeCursor();
        return $images;
    }

    private function findQuery( $keywords, $orderBy, $order )
    {
        $keywords = $this->pdo->quote( OnpubDatabase::utf8Decode( trim( $keywords ) ) );

        if ( $orderBy ) {
            if ( $order ) {
                return "SELECT ID, websiteID, fileName ,description, created, modified FROM OnpubImages WHERE ID RLIKE $keywords OR fileName RLIKE $keywords OR description RLIKE $keywords OR modified RLIKE $keywords ORDER BY $orderBy $order";
            }
            else {
                return "SELECT ID, websiteID, fileName ,description, created, modified FROM OnpubImages WHERE ID RLIKE $keywords OR fileName RLIKE $keywords OR description RLIKE $keywords OR modified RLIKE $keywords ORDER BY $orderBy";
            }
        }
        else {
            return "SELECT ID, websiteID, fileName ,description, created, modified FROM OnpubImages WHERE ID RLIKE $keywords OR fileName RLIKE $keywords OR description RLIKE $keywords OR modified RLIKE $keywords ORDER BY modified DESC";
        }
    }

    /**
     * @param OnpubImage $image The image to be updated.
     * @return int 1 if the image was updated. 0 if the image does not exist in the database.
     */
    public function update( OnpubImage $image )
    {
        $now = new OnpubDateTime();

        $stmt = $this->pdo->prepare( "UPDATE OnpubImages SET fileName = :fileName, description = :description, modified = :modified WHERE ID = :ID" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        $stmt->bindParam( ':fileName', OnpubDatabase::utf8Decode( trim( $image->getFileName() ) ) );
        $stmt->bindParam( ':description', OnpubDatabase::utf8Decode( trim( $image->getDescription() ) ) );
        $stmt->bindParam( ':modified', $now->toString() );
        $stmt->bindParam( ':ID', $image->getID() );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

        return $stmt->rowCount();
    }

    /**
     * @param int $ID ID of the image to delete.
     * @return int 1 if the image was deleted, 0 if the image does not exist in the database.
     */
    public function delete( $ID )
    {
        $stmt = $this->pdo->prepare( "DELETE FROM OnpubImages WHERE ID = :ID" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );

        $stmt->bindParam( ':ID', $ID );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, false, $stmt->errorInfo() );

        return $stmt->rowCount();
    }
}

?>
