<?php

/**
 * @author {@link mailto:corey@onpubco.com Corey HM Taylor}
 * @copyright Onpub (TM) Copyright (C) 2007, Onpubco
 * {@link http://onpubco.com/}
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * Version 2
 * @package onpubapi
 * @version 0.8.3
 */
class OnpubSAMaps
{
    private $pdo;
    /**
     * @var bool
     */
    public $enableTransactions;

    /**
     * @param PDO $pdo A {@link PHP_MANUAL#function.pdo-construct PHP Data Object}
     * @param bool $enableTransactions
     */
    function __construct( PDO $pdo, $enableTransactions = true )
    {
        $this->pdo = $pdo;
        $this->enableTransactions = $enableTransactions;
        $this->pdo->setAttribute( PDO::ATTR_EMULATE_PREPARES, true );
    }

    /**
     * @param int $sectionID Delete the map(s) with sectionID.
     * @param int $articleID Delete the map(s) with articleID.
     * @return int The number of maps deleted, 0 if no maps were deleted.
     */
    public function delete( $sectionID, $articleID )
    {
        if ( $sectionID && $articleID ) {
            $stmt = $this->pdo->prepare( "DELETE FROM OnpubSAMaps WHERE sectionID = :sectionID AND articleID = :articleID" );
            OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );
            $stmt->bindParam( ':sectionID', $sectionID );
            $stmt->bindParam( ':articleID', $articleID );
        }

        if ( $sectionID && !$articleID ) {
            $stmt = $this->pdo->prepare( "DELETE FROM OnpubSAMaps WHERE sectionID = :sectionID" );
            OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );
            $stmt->bindParam( ':sectionID', $sectionID );
        }

        if ( !$sectionID && $articleID ) {
            $stmt = $this->pdo->prepare( "DELETE FROM OnpubSAMaps WHERE articleID = :articleID" );
            OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );
            $stmt->bindParam( ':articleID', $articleID );
        }

        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, false, $stmt->errorInfo() );

        return $stmt->rowCount();
    }

    /**
     * @param OnpubSAMap $samap Map whose ID you want to get.
     * @return int The ID of the map. NULL if the map does not exist in the database.
     */
    public function getID( OnpubSAMap $samap )
    {
        $stmt = $this->pdo->prepare( "SELECT ID FROM OnpubSAMaps WHERE sectionID = :sectionID AND articleID = :articleID" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );

        $stmt->bindParam( ':sectionID', $samap->getSectionID() );
        $stmt->bindParam( ':articleID', $samap->getArticleID() );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, false, $stmt->errorInfo() );

        if ( ($row = $stmt->fetch( PDO::FETCH_ASSOC )) ) {
            return $row["ID"];
        }

        return null;
    }

    /**
     * @param OnpubQueryOptions $queryOptions Database query options.
     * @return array An array of {@link OnpubSAMap} objects.
     */
    public function select( OnpubQueryOptions $queryOptions, $sectionID = null, $articleID = null )
    {
        $query = $this->selectQuery( $queryOptions, $sectionID, $articleID );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );
        $rows = $result->fetchAll( PDO::FETCH_ASSOC );

        $samaps = array();

        if ( $rows ) {
            foreach ( $rows as $row ) {
                $samap = new OnpubSAMap();

                $samap->setID( $row["ID"] );
                $samap->setSectionID( $row["sectionID"] );
                $samap->setArticleID( $row["articleID"] );
                $samap->setCreated( new OnpubDateTime( $row["created"] ) );
                $samap->setModified( new OnpubDateTime( $row["modified"] ) );

                $samaps[] = $samap;
            }
        }

        $result->closeCursor();
        return $samaps;
    }

    private function selectQuery( OnpubQueryOptions $queryOptions, $sectionID = null, $articleID = null )
    {
        $where = "";
        $orderBy = "";
        $limit = "";

        if ( $queryOptions->getDateLimit() ) {
            $where = "WHERE created <= " . $this->pdo->quote( $queryOptions->getDateLimit()->toString() );

            if ( $sectionID ) {
                $where .= " AND sectionID = $sectionID";
            }
            elseif ( $articleID ) {
                $where .= " AND articleID = $articleID";
            }
        }
        else {
            if ( $sectionID ) {
                $where = "WHERE sectionID = $sectionID";
            }
            elseif ( $articleID ) {
                $where = "WHERE articleID = $articleID";
            }
        }

        if ( $queryOptions->getOrderBy() ) {
            $orderBy = "ORDER BY " . $queryOptions->getOrderBy();

            if ( $queryOptions->getOrder() ) {
                $orderBy .= " " . $queryOptions->getOrder();
            }
        }

        if ( $queryOptions->getPage() && $queryOptions->getRowLimit() && $queryOptions->getRowLimit() > 0 ) {
            $limit = "LIMIT " . (($queryOptions->getPage() - 1) * $queryOptions->getRowLimit()) . "," . $queryOptions->getRowLimit();
        }
        elseif ( $queryOptions->getRowLimit() && $queryOptions->getRowLimit() > 0 ) {
            $limit = "LIMIT 0," . $queryOptions->getRowLimit();
        }

        return "SELECT ID, sectionID, articleID, created, modified FROM OnpubSAMaps $where $orderBy $limit";
    }

    /**
     * @param mixed $samaps A single {@link OnpubSAMap} object or an array of {@link OnpubSAMap} objects (to insert multiple maps at a time).
     * @return mixed The ID(s) of the new map(s). An int will be returned if a single map was inserted. An array of ints will be returned if multiple maps were inserted.
     * @throws PDOException if there's a database error.
     */
    public function insert( $samaps )
    {
        $samapIDs = array();
        $isArray = true;

        if ( !is_array( $samaps ) ) {
            $samaps = array( $samaps );
            $isArray = false;
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        $stmt = $this->pdo->prepare( "INSERT INTO OnpubSAMaps (ID, sectionID, articleID, created, modified) VALUES (:ID, :sectionID, :articleID, :created, :modified)" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        foreach ( $samaps as $samap ) {
            try {
                $samapID = $this->getID( $samap );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }

            if ( $samapID ) {
                $samapIDs[] = $samapID;
                $samap->setID( $samapID );
            }
            else {
                $stmt->bindParam( ':ID', $samap->getID() );
                $stmt->bindParam( ':sectionID', $samap->getSectionID() );
                $stmt->bindParam( ':articleID', $samap->getArticleID() );
                $stmt->bindParam( ':created', $samap->getCreated()->toString() );
                $stmt->bindParam( ':modified', $samap->getModified()->toString() );
                $result = $stmt->execute();
                OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

                $samapIDs[] = $this->pdo->lastInsertId();
                $samap->setID( $this->pdo->lastInsertId() );
            }
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        if ( $isArray ) {
            return $samapIDs;
        }
        else {
            return end( $samapIDs );
        }
    }

    /**
     * @param OnpubSAMap $samap The map to be updated.
     * @return int 1 if the map was updated. 0 if the map does not exist in the database.
     */
    public function update( OnpubSAMap $samap )
    {
        $stmt = $this->pdo->prepare( "UPDATE OnpubSAMaps SET sectionID = :sectionID, articleID = :articleID, created = :created, modified = :modified WHERE ID = :ID" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        $stmt->bindParam( ':sectionID', $samap->getSectionID() );
        $stmt->bindParam( ':articleID', $samap->getArticleID() );
        $stmt->bindParam( ':created', $samap->getCreated()->toString() );
        $stmt->bindParam( ':modified', $samap->getModified()->toString() );
        $stmt->bindParam( ':ID', $samap->getID() );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

        return $stmt->rowCount();
    }
}

?>
