<?php

/**
 * @author {@link mailto:corey@onpubco.com Corey HM Taylor}
 * @copyright Onpub (TM) Copyright (C) 2007, Onpubco
 * {@link http://onpubco.com/}
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * Version 2
 * @package onpubapi
 * @version 0.8.3
 */
class OnpubSections
{
    private $pdo;
    /**
     * @var bool
     */
    public $enableTransactions;

    /**
     * @param PDO $pdo A {@link PHP_MANUAL#function.pdo-construct PHP Data Object}
     * @param bool $enableTransactions
     */
    function __construct( PDO $pdo, $enableTransactions = true )
    {
        $this->pdo = $pdo;
        $this->enableTransactions = $enableTransactions;
        $this->pdo->setAttribute( PDO::ATTR_EMULATE_PREPARES, true );
    }

    /**
     * @return int
     */
    public function count()
    {
        $query = $this->countQuery();
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );

        if ( !($row = $result->fetch( PDO::FETCH_ASSOC )) ) {
            return 0;
        }

        $result->closeCursor();
        return $row["count"];
    }

    private function countQuery()
    {
        return "SELECT COUNT(ID) AS count FROM OnpubSections";
    }

    /**
     * @param int $ID ID of the section to delete.
     * @return int 1 if the section was deleted, 0 if the section does not exist in the database.
     */
    public function delete( $ID )
    {
        $osamaps = new OnpubSAMaps( $this->pdo, false );
        $owsmaps = new OnpubWSMaps( $this->pdo, false );

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        try {
            $osamaps->delete( $ID, null );
        }
        catch ( PDOException $e ) {
            if ( $this->enableTransactions ) $this->pdo->rollBack();
            throw $e;
        }

        try {
            $owsmaps->delete( null, $ID );
        }
        catch ( PDOException $e ) {
            if ( $this->enableTransactions ) $this->pdo->rollBack();
            throw $e;
        }

        $stmt = $this->pdo->prepare( "DELETE FROM OnpubSections WHERE ID = :ID" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        $stmt->bindParam( ':ID', $ID );

        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        return $stmt->rowCount();
    }

    /**
     * @param string $keywords Keyword(s) to search for.
     * @return array All the sections which were found as an array of {@link OnpubSection} objects.
     */
    public function search( $keywords, $orderBy = null, $order = null )
    {
        $query = $this->findQuery( $keywords, $orderBy, $order );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );

        $rows = $result->fetchAll( PDO::FETCH_ASSOC );

        $sections = array();

        if ( $rows ) {
            foreach ( $rows as $row ) {
                $section = new OnpubSection();

                $section->setID( $row["ID"] );
                $section->setImageID( $row["imageID"] );
                $section->setWebsiteID( $row["websiteID"] );
                $section->setParentID( $row["parentID"] );
                $section->setName( stripslashes( $row["name"] ) );
                $section->setCreated( new OnpubDateTime( $row["created"] ) );
                $section->setModified( new OnpubDateTime( $row["modified"] ) );

                $sections[] = $section;
            }
        }

        $result->closeCursor();
        return $sections;
    }

    private function findQuery( $keywords, $orderBy, $order )
    {
        $keywords = $this->pdo->quote( OnpubDatabase::utf8Decode( trim( $keywords ) ) );

        if ( $orderBy ) {
            if ( $order ) {
                return "SELECT ID, imageID, websiteID, parentID, name, created, modified FROM OnpubSections WHERE ID RLIKE $keywords OR name RLIKE $keywords OR modified RLIKE $keywords AND parentID IS NULL ORDER BY $orderBy $order";
            }
            else {
                return "SELECT ID, imageID, websiteID, parentID, name, created, modified FROM OnpubSections WHERE ID RLIKE $keywords OR name RLIKE $keywords OR modified RLIKE $keywords AND parentID IS NULL ORDER BY $orderBy";
            }
        }
        else {
            return "SELECT ID, imageID, websiteID, parentID, name, created, modified FROM OnpubSections WHERE ID RLIKE $keywords OR name RLIKE $keywords OR modified RLIKE $keywords AND parentID IS NULL ORDER BY modified DESC";
        }
    }

    /**
     * @param OnpubQueryOptions $queryOptions Database query options.
     * @return array An array of {@link OnpubSection} objects.
     */
    public function select( OnpubQueryOptions $queryOptions, $websiteID = null, $flat = true, $parentID = null )
    {
        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        $query = $this->selectQuery( $queryOptions, $websiteID, $parentID );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, $this->enableTransactions );
        $rows = $result->fetchAll( PDO::FETCH_ASSOC );

        $sections = array();

        if ( $rows ) {
            foreach ( $rows as $row ) {
                $section = new OnpubSection();

                $section->setID( $row["ID"] );
                $section->setImageID( $row["imageID"] );
                $section->setWebsiteID( $row["websiteID"] );
                $section->setParentID( $row["parentID"] );
                $section->setName( stripslashes( $row["name"] ) );
                $section->setCreated( new OnpubDateTime( $row["created"] ) );
                $section->setModified( new OnpubDateTime( $row["modified"] ) );

                $sections[] = $section;
            }
        }

        $sectionsassoc = array();

        for ( $i = 0; $i < sizeof( $sections ); $i++ ) {
            $sectionsassoc[$sections[$i]->getID()] = $sections[$i];
        }

        for ( $i = 0; $i < sizeof( $sections ); $i++ ) {
            $section = $sections[$i];
            $parent = null;

            while ( $section->getParentID() ) {
                if ( isset( $sectionsassoc[$section->getParentID()] ) ) {
                    $parent = $sectionsassoc[$section->getParentID()];
                }
                else {
                    // Here's where we have to lookup section from DB.
                    $query = $this->getQuery( $section->getParentID(), false, false, null, null, null );
                    $result = $this->pdo->query( $query );
                    OnpubDatabase::verifyQuery( $this->pdo, $result, $this->enableTransactions );

                    $row = $result->fetch( PDO::FETCH_ASSOC );

                    $parent = new OnpubSection();

                    if ( $row ) {
                        $parent->setSectionID( $row["ID"] );
                        $parent->setImageID( $row["imageID"] );
                        $parent->setWebsiteID( $row["websiteID"] );
                        $parent->setParentID( $row["parentID"] );
                        $parent->setName( stripslashes( $row["name"] ) );
                        $parent->setCreated( new OnpubDateTime( $row["created"] ) );
                        $parent->setModified( new OnpubDateTime( $row["modified"] ) );

                        $result->closeCursor();
                    }

                    $sectionsassoc[$parent->getSectionID()] = $parent;
                }

                $section->setParent( $parent );

                $found = false;

                foreach ( $parent->getSections() as $s ) {
                    if ( $s->getID() == $section->getID() ) {
                        $found = true;
                    }
                }

                if ( !$found ) {
                    $parent->addSection( $section );
                }

                $section = $parent;
            }
        }

        $sections = array();

        foreach ( $sectionsassoc as $s ) {
            if ( $flat ) {
                $sections[] = $s;
            }
            else {
                if ( !$s->getParentID() ) {
                    $sections[] = $s;
                }
            }
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        if ( $parentID ) {
            if ( sizeof( $sections ) ) {
                return $sections[0]->getSections();
            }
        }

        $result->closeCursor();
        return $sections;
    }

    private function selectQuery( OnpubQueryOptions $queryOptions, $websiteID, $parentID )
    {
        $where = "";
        $orderBy = "";
        $limit = "";

        if ( $queryOptions->getDateLimit() ) {
            $where = "WHERE created <= " . $this->pdo->quote( $queryOptions->getDateLimit()->toString() );

            if ( $websiteID ) {
                $where .= " AND websiteID = $websiteID";
            }
            elseif ( $parentID ) {
                $where .= " AND parentID = $parentID";
            }
        }
        else {
            if ( $websiteID ) {
                $where = "WHERE websiteID = $websiteID";
            }
            elseif ( $parentID ) {
                $where = "WHERE parentID = $parentID";
            }
        }

        if ( $queryOptions->getOrderBy() ) {
            $orderBy = "ORDER BY " . $queryOptions->getOrderBy();

            if ( $queryOptions->getOrder() ) {
                $orderBy .= " " . $queryOptions->getOrder();
            }
        }

        if ( $queryOptions->getPage() && $queryOptions->getRowLimit() && $queryOptions->getRowLimit() > 0 ) {
            $limit = "LIMIT " . (($queryOptions->getPage() - 1) * $queryOptions->getRowLimit()) . "," . $queryOptions->getRowLimit();
        }
        elseif ( $queryOptions->getRowLimit() && $queryOptions->getRowLimit() > 0 ) {
            $limit = "LIMIT 0," . $queryOptions->getRowLimit();
        }

        return "SELECT ID, imageID, websiteID, parentID, name, created, modified FROM OnpubSections $where $orderBy $limit";
    }

    /**
     * @param int $ID ID of the section to get.
     * @return OnpubSection An {@link OnpubSection} object. NULL if the section does not exist in the database.
     */
    public function get( $ID, $join = false, $includeContent = true, $orderBy = null, $order = null, $dateLimit = null )
    {
        $query = $this->getQuery( $ID, $join, $includeContent, $orderBy, $order, $dateLimit );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );

        if ( !($rows = $result->fetchAll( PDO::FETCH_ASSOC )) ) {
            return null;
        }

        $row = $rows[0];

        $section = new OnpubSection();

        $section->setID( $row["ID"] );
        $section->setImageID( $row["imageID"] );
        $section->setWebsiteID( $row["websiteID"] );
        $section->setParentID( $row["parentID"] );
        $section->setName( stripslashes( $row["name"] ) );
        $section->setCreated( new OnpubDateTime( $row["created"] ) );
        $section->setModified( new OnpubDateTime( $row["modified"] ) );

        if (!$join) {
            return $section;
        }

        if ( $row["imageID"] ) {
            $image = new OnpubImage();

            $image->setID( $row["imageID"] );
            $image->setWebsiteID( $row["websiteID"] );
            $image->setFileName( stripslashes( $row["fileName"] ) );
            $image->setDescription( stripslashes( $row["description"] ) );
            $image->setCreated( new OnpubDateTime( $row["imageCreated"] ) );
            $image->setModified( new OnpubDateTime( $row["imageModified"] ) );

            $section->setImage( $image );
        }

        $articlesassoc = array();

        foreach ( $rows as $row ) {
            $articleID = $row["articleID"];

            if ( $articleID ) {
                if ( $includeContent ) {
                    $content = stripslashes( $row["content"] );
                }
                else {
                    $content = "";
                }

                $article = new OnpubArticle();

                $article->setID( $articleID );
                $article->setImageID( $row["articleImageID"] );
                $article->setTitle( stripslashes( $row["title"] ) );
                $article->setContent( $content );
                $article->setURL( stripslashes( $row["articleURL"] ) );
                $article->setCreated( new OnpubDateTime( $row["articleCreated"] ) );
                $article->setModified( new OnpubDateTime( $row["articleModified"] ) );

                if ( isset( $articlesassoc["$articleID"] ) ) {
                    $article = $articlesassoc["$articleID"];
                }
                else {
                    $articlesassoc["$articleID"] = $article;
                }

                if ( $row["articleImageID"] ) {
                    $image = new OnpubImage();

                    $image->setID( $row["articleImageID"] );
                    $image->setWebsiteID( $row["articleImageWebsiteID"] );
                    $image->setFileName( stripslashes( $row["articleImageFileName"] ) );
                    $image->setDescription( stripslashes( $row["articleImageDescription"] ) );
                    $image->setCreated( new OnpubDateTime( $row["articleImageCreated"] ) );
                    $image->setModified( new OnpubDateTime( $row["articleImageModified"] ) );

                    $article->setImage( $image );
                }

                $articles = $section->getArticles();
                $exists = false;

                foreach ( $articles as $a ) {
                    if ( $a->getID() == $article->getID() ) {
                        $exists = true;
                    }
                }

                if ( $exists == false ) {
                    $articles[] = $article;
                    $section->setArticles( $articles );
                }
            }
        }

        $authorsassoc = array();

        reset( $rows );
        foreach ( $rows as $row ) {
            $authorID = $row["authorID"];
            $articleID = $row["articleID"];

            if ( $authorID && $articleID ) {
                $author = new OnpubAuthor();

                $author->setID( $authorID );
                $author->setImageID( $row["authorImageID"] );
                $author->setGivenNames( stripslashes( $row["givenNames"] ) );
                $author->setFamilyName( stripslashes( $row["familyName"] ) );
                $author->setDisplayAs( stripslashes( $row["displayAs"] ) );
                $author->setCreated( new OnpubDateTime( $row["authorCreated"] ) );
                $author->setModified( new OnpubDateTime( $row["authorModified"] ) );

                if ( isset( $authorsassoc["$authorID"] ) ) {
                    $author = $authorsassoc["$authorID"];
                }
                else {
                    $authorsassoc["$authorID"] = $author;
                }

                if ( $row["authorImageID"] ) {
                    $image = new OnpubImage();

                    $image->setID( $row["authorImageID"] );
                    $image->setWebsiteID( $row["authorImageWebsiteID"] );
                    $image->setFileName( stripslashes( $row["authorImageFileName"] ) );
                    $image->setDescription( stripslashes( $row["authorImageDescription"] ) );
                    $image->setCreated( new OnpubDateTime( $row["authorImageCreated"] ) );
                    $image->setModified( new OnpubDateTime( $row["authorImageModified"] ) );

                    $author->setImage( $image );
                }

                $article = $articlesassoc["$articleID"];
                $authors = $article->getAuthors();
                $exists = false;

                foreach ( $authors as $a ) {
                    if ( $a->getID() == $author->getID() ) {
                        $exists = true;
                    }
                }

                if ( $exists == false ) {
                    $authors[] = $author;
                    $article->setAuthors( $authors );
                }
            }
        }

        $result->closeCursor();
        return $section;
    }

    private function getQuery( $ID, $join, $includeContent, $orderBy, $order, $dateLimit )
    {
        if ( $orderBy ) {
            $orderBy = "articles.$orderBy";
        }
        else {
            $orderBy = "samaps.ID";
        }

        if ( !$order ) {
            $order = "ASC";
        }

        $where = "WHERE sections.ID = $ID";

        if ( $dateLimit ) {
            $where .= " AND articles.created <= '$dateLimit'";
        }

        if ( $includeContent ) {
            $articleColumns = "articles.ID AS articleID, articles.imageID AS articleImageID, title, content, articles.url AS articleURL, articles.created AS articleCreated, articles.modified AS articleModified";
        }
        else {
            $articleColumns = "articles.ID AS articleID, articles.imageID AS articleImageID, title, articles.url AS articleURL, articles.created AS articleCreated, articles.modified AS articleModified";
        }

        if ( $join ) {
            return "SELECT sections.ID, sections.imageID, sections.websiteID, sections.parentID, sections.name, sections.url, sections.created, sections.modified, sectionimages.fileName, sectionimages.description, sectionimages.created AS imageCreated, sectionimages.modified AS imageModified, $articleColumns, articleimages.websiteID AS articleImageWebsiteID, articleimages.fileName AS articleImageFileName, articleimages.description AS articleImageDescription, articleimages.created AS articleImageCreated, articleimages.modified AS articleImageModified, authors.ID AS authorID, authors.imageID AS authorImageID, authors.givenNames, authors.familyName, authors.displayAs, authors.created AS authorCreated, authors.modified AS authorModified, authorimages.websiteID AS authorImageWebsiteID, authorimages.fileName AS authorImageFileName, authorimages.description AS authorImageDescription, authorimages.created AS authorImageCreated, authorimages.modified AS authorImageModified FROM OnpubSections AS sections LEFT JOIN OnpubImages AS sectionimages ON sections.imageID = sectionimages.ID LEFT JOIN OnpubSAMaps AS samaps ON sections.ID = samaps.sectionID LEFT JOIN OnpubArticles AS articles ON samaps.articleID = articles.ID LEFT JOIN OnpubImages AS articleimages ON articles.imageID = articleimages.ID LEFT JOIN OnpubAAMaps AS aamaps ON articles.ID = aamaps.articleID LEFT JOIN OnpubAuthors AS authors ON aamaps.authorID = authors.ID LEFT JOIN OnpubImages AS authorimages ON authors.imageID = authorimages.ID $where ORDER BY $orderBy $order";
        }
        else {
            return "SELECT ID, imageID, websiteID, parentID, name, created, modified FROM OnpubSections WHERE ID = $ID";
        }
    }

    /**
     * @param OnpubSection $section Section whose ID you want to get.
     * @return int The ID of the section. NULL if the section does not exist in the database.
     */
    public function getID( OnpubSection $section )
    {
        if ( $section->getParentID() ) {
            $stmt = $this->pdo->prepare( "SELECT ID FROM OnpubSections WHERE websiteID = :websiteID AND parentID = :parentID AND name = :name" );
        }
        else {
            $stmt = $this->pdo->prepare( "SELECT ID FROM OnpubSections WHERE websiteID = :websiteID AND parentID IS NULL AND name = :name" );
        }

        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );

        $stmt->bindParam( ':websiteID', $section->getWebsiteID() );

        if ( $section->getParentID() ) {
            $stmt->bindParam( ':parentID', $section->getParentID() );
        }

        $stmt->bindParam( ':name', OnpubDatabase::utf8Decode( trim( $section->getName() ) ) );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, false, $stmt->errorInfo() );

        if ( ($row = $stmt->fetch( PDO::FETCH_ASSOC )) ) {
            return $row["ID"];
        }

        return null;
    }

    /**
     * @param mixed $sections A single {@link OnpubSection} object or an array of {@link OnpubSection} objects (to insert multiple sections at a time).
     * @return mixed The ID(s) of the new section(s). An int will be returned if a single section was inserted. An array of ints will be returned if multiple sections were inserted.
     * @throws PDOException if there's a database error.
     */
    public function insert( $sections, $insertMaps = false )
    {
        $oimages = new OnpubImages( $this->pdo, false );
        $owsmaps = new OnpubWSMaps( $this->pdo, false );
        $IDs = array();
        $isArray = true;
        $wsmaps = array();

        if ( !is_array( $sections ) ) {
            $sections = array( $sections );
            $isArray = false;
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        $stmt = $this->pdo->prepare( "INSERT INTO OnpubSections (ID, imageID, websiteID, parentID, name, created, modified) VALUES (:ID, :imageID, :websiteID, :parentID, :name, :created, :modified)" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        foreach ( $sections as $section ) {
            if ( $section->getImage() ) {
                try {
                    $imageID = $oimages->insert( $section->getImage() );
                    $section->setImageID( $imageID );
                }
                catch ( PDOException $e ) {
                    if ( $this->enableTransactions ) $this->pdo->rollBack();
                    throw $e;
                }
            }

            try {
                $ID = $this->getID( $section );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }

            if ( $ID ) {
                $IDs[] = $ID;
                $section->setID( $ID );
            }
            else {
                $stmt->bindParam( ':ID', $section->getID() );
                $stmt->bindParam( ':imageID', $section->getImageID() );
                $stmt->bindParam( ':websiteID', $section->getWebsiteID() );
                $stmt->bindParam( ':parentID', $section->getParentID() );
                $stmt->bindParam( ':name', OnpubDatabase::utf8Decode( trim( $section->getName() ) ) );
                $stmt->bindParam( ':created', $section->getCreated()->toString() );
                $stmt->bindParam( ':modified', $section->getModified()->toString() );
                $result = $stmt->execute();
                OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

                $IDs[] = $this->pdo->lastInsertId();
                $section->setID( $this->pdo->lastInsertId() );
            }

            $wsmap = new OnpubWSMap();

            $wsmap->setWebsiteID( $section->getWebsiteID() );
            $wsmap->setSectionID( $section->getID() );
            $wsmap->setCreated( $section->getCreated() );
            $wsmap->setModified( $section->getModified() );

            $wsmaps[] = $wsmap;
        }

        if ( $insertMaps ) {
            try {
                $owsmaps->insert( $wsmaps );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        if ( $isArray ) {
            return $IDs;
        }
        else {
            return end( $IDs );
        }
    }

    /**
     * @param OnpubSection $section The section to be updated.
     * @return int 1 if the section was updated. 0 if the section does not exist in the database.
     */
    public function update( OnpubSection $section )
    {
        $oarticles = new OnpubArticles( $this->pdo, false );
        $osamaps = new OnpubSAMaps( $this->pdo, false );
        $now = new OnpubDateTime();

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        $stmt = $this->pdo->prepare( "UPDATE OnpubSections SET imageID = :imageID, name = :name, modified = :modified WHERE ID = :ID" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        $stmt->bindParam( ':imageID', $section->getImageID() );
        $stmt->bindParam( ':name', OnpubDatabase::utf8Decode( trim( $section->getName() ) ) );
        $stmt->bindParam( ':modified', $now->toString() );
        $stmt->bindParam( ':ID', $section->getID() );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

        try {
            $osamaps->delete( $section->getID(), null );
        }
        catch ( PDOException $e ) {
            if ( $this->enableTransactions ) $this->pdo->rollBack();
            throw $e;
        }

        $articles = $section->getArticles();
        $samaps = array();

        foreach ( $articles as $article ) {
            if ( $article->getID() ) {
                try {
                    $article = $oarticles->get( $article->getID(), new OnpubQueryOptions() );
                }
                catch ( PDOException $e ) {
                    if ( $this->enableTransactions ) $this->pdo->rollBack();
                    throw $e;
                }

                $samap = new OnpubSAMap();

                $samap->setSectionID( $section->getID() );
                $samap->setArticleID( $article->getID() );
                $samap->setCreated( $article->getCreated() );
                $samap->setModified( $article->getModified() );

                $samaps[] = $samap;
            }
            else {
                try {
                    $articleID = $oarticles->insert( $article );
                }
                catch ( PDOException $e ) {
                    if ( $this->enableTransactions ) $this->pdo->rollBack();
                    throw $e;
                }

                $samap = new OnpubSAMap();

                $samap->setSectionID( $section->getID() );
                $samap->setArticleID( $articleID );

                $samaps[] = $samap;
            }
        }

        try {
            $osamaps->insert( $samaps );
        }
        catch ( PDOException $e ) {
            if ( $this->enableTransactions ) $this->pdo->rollBack();
            throw $e;
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        return $stmt->rowCount();
    }
}

?>
