<?php

/**
 * @author {@link mailto:corey@onpubco.com Corey HM Taylor}
 * @copyright Onpub (TM) Copyright (C) 2007, Onpubco
 * {@link http://onpubco.com/}
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * Version 2
 * @package onpubapi
 * @version 0.8.3
 */
class OnpubWSMaps
{
    private $pdo;
    /**
     * @var bool
     */
    public $enableTransactions;

    /**
     * @param PDO $pdo A {@link PHP_MANUAL#function.pdo-construct PHP Data Object}
     * @param bool $enableTransactions
     */
    function __construct( PDO $pdo, $enableTransactions = true )
    {
        $this->pdo = $pdo;
        $this->enableTransactions = $enableTransactions;
        $this->pdo->setAttribute( PDO::ATTR_EMULATE_PREPARES, true );
    }

    /**
     * @param int $websiteID Delete the map(s) with websiteID.
     * @param int $sectionID Delete the map(s) with sectionID.
     * @return int The number of maps deleted, 0 if no maps were deleted.
     */
    public function delete( $websiteID, $sectionID )
    {
        if ( $websiteID && $sectionID ) {
            $stmt = $this->pdo->prepare( "DELETE FROM OnpubWSMaps WHERE websiteID = :websiteID AND sectionID = :sectionID" );
            OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );
            $stmt->bindParam( ':websiteID', $websiteID );
            $stmt->bindParam( ':sectionID', $sectionID );
        }

        if ( $websiteID && !$sectionID ) {
            $stmt = $this->pdo->prepare( "DELETE FROM OnpubWSMaps WHERE websiteID = :websiteID" );
            OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );
            $stmt->bindParam( ':websiteID', $websiteID );
        }

        if ( !$websiteID && $sectionID ) {
            $stmt = $this->pdo->prepare( "DELETE FROM OnpubWSMaps WHERE sectionID = :sectionID" );
            OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );
            $stmt->bindParam( ':sectionID', $sectionID );
        }

        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, false, $stmt->errorInfo() );

        return $stmt->rowCount();
    }

    /**
     * @param OnpubWSMap $wsmap Map whose ID you want to get.
     * @return int The ID of the map. NULL if the map does not exist in the database.
     */
    public function getID( OnpubWSMap $wsmap )
    {
        $stmt = $this->pdo->prepare( "SELECT ID FROM OnpubWSMaps WHERE websiteID = :websiteID AND sectionID = :sectionID" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );

        $stmt->bindParam( ':websiteID', $wsmap->getWebsiteID() );
        $stmt->bindParam( ':sectionID', $wsmap->getSectionID() );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, false, $stmt->errorInfo() );

        if ( ($row = $stmt->fetch( PDO::FETCH_ASSOC )) ) {
            return $row["ID"];
        }

        return null;
    }

    /**
     * @param OnpubQueryOptions $queryOptions Database query options.
     * @return array An array of {@link OnpubWSMap} objects.
     */
    public function select( OnpubQueryOptions $queryOptions, $websiteID = null )
    {
        $query = $this->selectQuery( $queryOptions, $websiteID );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );
        $rows = $result->fetchAll( PDO::FETCH_ASSOC );

        $wsmaps = array();

        if ( $rows ) {
            foreach ( $rows as $row ) {
                $wsmap = new OnpubWSMap();

                $wsmap->setID( $row["ID"] );
                $wsmap->setWebsiteID( $row["websiteID"] );
                $wsmap->setSectionID( $row["sectionID"] );
                $wsmap->setCreated( new OnpubDateTime( $row["created"] ) );
                $wsmap->setModified( new OnpubDateTime( $row["modified"] ) );

                $wsmaps[] = $wsmap;
            }
        }

        $result->closeCursor();
        return $wsmaps;
    }

    private function selectQuery( OnpubQueryOptions $queryOptions, $websiteID )
    {
        $where = "";
        $orderBy = "";
        $limit = "";

        if ( $queryOptions->getDateLimit() ) {
            $where = "WHERE created <= " . $this->pdo->quote( $queryOptions->getDateLimit()->toString() );

            if ( $websiteID ) {
                $where .= " AND websiteID = $websiteID";
            }
        }
        else {
            if ( $websiteID ) {
                $where = "WHERE websiteID = $websiteID";
            }
        }

        if ( $queryOptions->getOrderBy() ) {
            $orderBy = "ORDER BY " . $queryOptions->getOrderBy();

            if ( $queryOptions->getOrder() ) {
                $orderBy .= " " . $queryOptions->getOrder();
            }
        }

        if ( $queryOptions->getPage() && $queryOptions->getRowLimit() && $queryOptions->getRowLimit() > 0 ) {
            $limit = "LIMIT " . (($queryOptions->getPage() - 1) * $queryOptions->getRowLimit()) . "," . $queryOptions->getRowLimit();
        }
        elseif ( $queryOptions->getRowLimit() && $queryOptions->getRowLimit() > 0 ) {
            $limit = "LIMIT 0," . $queryOptions->getRowLimit();
        }

        return "SELECT ID, websiteID, sectionID, created, modified FROM OnpubWSMaps $where $orderBy $limit";
    }

    /**
     * @param mixed $wsmaps A single {@link OnpubWSMap} object or an array of {@link OnpubWSMap} objects (to insert multiple maps at a time).
     * @return mixed The ID(s) of the new map(s). An int will be returned if a single map was inserted. An array of ints will be returned if multiple maps were inserted.
     * @throws PDOException if there's a database error.
     */
    public function insert( $wsmaps )
    {
        $wsmapIDs = array();
        $isArray = true;

        if ( !is_array( $wsmaps ) ) {
            $wsmaps = array( $wsmaps );
            $isArray = false;
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        $stmt = $this->pdo->prepare( "INSERT INTO OnpubWSMaps (ID, websiteID, sectionID, created, modified) VALUES (:ID, :websiteID, :sectionID, :created, :modified)" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        foreach ( $wsmaps as $wsmap ) {
            try {
                $wsmapID = $this->getID( $wsmap );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }

            if ( $wsmapID ) {
                $wsmapIDs[] = $wsmapID;
                $wsmap->setID( $wsmapID );
            }
            else {
                $stmt->bindParam( ':ID', $wsmap->getID() );
                $stmt->bindParam( ':websiteID', $wsmap->getWebsiteID() );
                $stmt->bindParam( ':sectionID', $wsmap->getSectionID() );
                $stmt->bindParam( ':created', $wsmap->getCreated()->toString() );
                $stmt->bindParam( ':modified', $wsmap->getModified()->toString() );
                $result = $stmt->execute();
                OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

                $wsmapIDs[] = $this->pdo->lastInsertId();
                $wsmap->setID( $this->pdo->lastInsertId() );
            }
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        if ( $isArray ) {
            return $wsmapIDs;
        }
        else {
            return end( $wsmapIDs );
        }
    }
}

?>
