<?php

/**
 * @author {@link mailto:corey@onpubco.com Corey HM Taylor}
 * @copyright Onpub (TM) Copyright (C) 2007, Onpubco
 * {@link http://onpubco.com/}
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * Version 2
 * @package onpubapi
 * @version 0.8.3
 */
class OnpubWebsites
{
    private $pdo;
    /**
     * @var bool
     */
    public $enableTransactions;

    /**
     * @param PDO $pdo A {@link PHP_MANUAL#function.pdo-construct PHP Data Object}
     * @param bool $enableTransactions
     */
    function __construct( PDO $pdo, $enableTransactions = true )
    {
        $this->pdo = $pdo;
        $this->enableTransactions = $enableTransactions;
        $this->pdo->setAttribute( PDO::ATTR_EMULATE_PREPARES, true );
    }

    /**
     * @return int
     */
    public function count()
    {
        $query = $this->countQuery();
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );

        if ( !($row = $result->fetch( PDO::FETCH_ASSOC )) ) {
            return 0;
        }

        $result->closeCursor();
        return $row["count"];
    }

    private function countQuery()
    {
        return "SELECT COUNT(ID) AS count FROM OnpubWebsites";
    }

    /**
     * @param int $ID ID of the website to delete.
     * @return int 1 if the website was deleted, 0 if the website does not exist in the database.
     */
    public function delete( $ID )
    {
        $stmt = $this->pdo->prepare( "DELETE FROM OnpubWebsites WHERE ID = :ID" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );

        $stmt->bindParam( ':ID', $ID );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, false, $stmt->errorInfo() );

        return $stmt->rowCount();
    }

    /**
     * @param string $keywords Keyword(s) to search for.
     * @return array All the websites which were found as an array of {@link OnpubWebsite} objects.
     */
    public function search( $keywords, $orderBy = null, $order = null )
    {
        $query = $this->findQuery( $keywords, $orderBy, $order );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );

        $rows = $result->fetchAll( PDO::FETCH_ASSOC );

        $websites = array();

        if ( $rows ) {
            foreach ( $rows as $row ) {
                $website = new OnpubWebsite();

                $website->setID( $row["ID"] );
                $website->setImageID( $row["imageID"] );
                $website->setName( stripslashes( $row["name"] ) );
                $website->setURL( stripslashes( $row["url"] ) );
                $website->setImagesURL( stripslashes( $row["imagesURL"] ) );
                $website->setImagesDirectory( stripslashes( $row["imagesDirectory"] ) );
                $website->setCreated( new OnpubDateTime( $row["created"] ) );
                $website->setModified( new OnpubDateTime( $row["modified"] ) );

                $websites[] = $website;
            }
        }

        $result->closeCursor();
        return $websites;
    }

    private function findQuery( $keywords, $orderBy, $order )
    {
        $keywords = $this->pdo->quote( OnpubDatabase::utf8Decode( trim( $keywords ) ) );

        if ( $orderBy ) {
            if ( $order ) {
                return "SELECT ID, imageID, name, url, imagesURL, imagesDirectory, created, modified FROM OnpubWebsites WHERE ID RLIKE $keywords OR name RLIKE $keywords OR modified RLIKE $keywords ORDER BY $orderBy $order";
            }
            else {
                return "SELECT ID, imageID, name, url, imagesURL, imagesDirectory, created, modified FROM OnpubWebsites WHERE ID RLIKE $keywords OR name RLIKE $keywords OR modified RLIKE $keywords ORDER BY $orderBy";
            }
        }
        else {
            return "SELECT ID, imageID, name, url, imagesURL, imagesDirectory, created, modified FROM OnpubWebsites WHERE ID RLIKE $keywords OR name RLIKE $keywords OR modified RLIKE $keywords ORDER BY modified DESC";
        }
    }

    /**
     * @param OnpubQueryOptions $queryOptions Database query options.
     * @return array An array of {@link OnpubWebsite} objects.
     */
    public function select( OnpubQueryOptions $queryOptions )
    {
        $query = $this->selectQuery( $queryOptions );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );
        $rows = $result->fetchAll( PDO::FETCH_ASSOC );

        $websites = array();

        if ( $rows ) {
            foreach ( $rows as $row ) {
                $website = new OnpubWebsite();

                $website->setID( $row["ID"] );
                $website->setImageID( $row["imageID"] );
                $website->setName( stripslashes( $row["name"] ) );
                $website->setURL( stripslashes( $row["url"] ) );
                $website->setImagesURL( stripslashes( $row["imagesURL"] ) );
                $website->setImagesDirectory( stripslashes( $row["imagesDirectory"] ) );
                $website->setCreated( new OnpubDateTime( $row["created"] ) );
                $website->setModified( new OnpubDateTime( $row["modified"] ) );

                $websites[] = $website;
            }
        }

        $result->closeCursor();
        return $websites;
    }

    private function selectQuery( OnpubQueryOptions $queryOptions )
    {
        $where = "";
        $orderBy = "";
        $limit = "";

        if ( $queryOptions->getDateLimit() ) {
            $where = "WHERE created <= " . $this->pdo->quote( $queryOptions->getDateLimit()->toString() );
        }

        if ( $queryOptions->getOrderBy() ) {
            $orderBy = "ORDER BY " . $queryOptions->getOrderBy();

            if ( $queryOptions->getOrder() ) {
                $orderBy .= " " . $queryOptions->getOrder();
            }
        }

        if ( $queryOptions->getPage() && $queryOptions->getRowLimit() && $queryOptions->getRowLimit() > 0 ) {
            $limit = "LIMIT " . (($queryOptions->getPage() - 1) * $queryOptions->getRowLimit()) . "," . $queryOptions->getRowLimit();
        }
        elseif ( $queryOptions->getRowLimit() && $queryOptions->getRowLimit() > 0 ) {
            $limit = "LIMIT 0," . $queryOptions->getRowLimit();
        }

        return "SELECT ID, imageID, ID, name, url, imagesURL, imagesDirectory, created, modified FROM OnpubWebsites $where $orderBy $limit";
    }

    /**
     * @param int $ID ID of the website to get.
     * @return OnpubWebsite An {@link OnpubWebsite} object. NULL if the website does not exist in the database.
     */
    public function get( $ID, $join = false, $includeContent = true, $orderBy = null, $order = null, $dateLimit = null, $dateAfter = null, $flatArticleList = true, $joinArticles = true )
    {
        $query = $this->getQuery( $ID, $join, $includeContent, $orderBy, $order, $dateLimit, $dateAfter, $joinArticles );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );

        if ( !($rows = $result->fetchAll( PDO::FETCH_ASSOC )) ) {
            return null;
        }

        $row = $rows[0];

        $website = new OnpubWebsite();

        $website->setID( $row["ID"] );
        $website->setImageID( $row["imageID"] );
        $website->setName( stripslashes( $row["name"] ) );
        $website->setURL( stripslashes( $row["url"] ) );
        $website->setImagesURL( stripslashes( $row["imagesURL"] ) );
        $website->setImagesDirectory( stripslashes( $row["imagesDirectory"] ) );
        $website->setCreated( new OnpubDateTime( $row["created"] ) );
        $website->setModified( new OnpubDateTime( $row["modified"] ) );

        if (!$join) {
            return $website;
        }

        if ( $row["imageID"] ) {
            $image = new OnpubImage();

            $image->setID( $row["imageID"] );
            $image->setWebsiteID( $row["ID"] );
            $image->setFileName( stripslashes( $row["fileName"] ) );
            $image->setDescription( stripslashes( $row["description"] ) );
            $image->setCreated( new OnpubDateTime( $row["imageCreated"] ) );
            $image->setModified( new OnpubDateTime( $row["imageModified"] ) );

            $website->setImage( $image );
        }

        $sectionsassoc = array();

        foreach ( $rows as $row ) {
            $sectionID = $row["sectionID"];
            $ID = $row["ID"];

            if ( $sectionID ) {
                $section = new OnpubSection();

                $section->setID( $sectionID );
                $section->setImageID( $row["sectionImageID"] );
                $section->setWebsiteID( $row["sectionWebsiteID"] );
                $section->setName( stripslashes( $row["sectionName"] ) );
                $section->setCreated( new OnpubDateTime( $row["sectionCreated"] ) );
                $section->setModified( new OnpubDateTime( $row["sectionModified"] ) );

                if ( isset( $sectionsassoc["$sectionID"] ) ) {
                    $section = $sectionsassoc["$sectionID"];
                }
                else {
                    $sectionsassoc["$sectionID"] = $section;
                }

                if ( $row["sectionImageID"] ) {
                    $image = new OnpubImage();

                    $image->setID( $row["sectionImageID"] );
                    $image->setWebsiteID( $row["sectionImageWebsiteID"] );
                    $image->setFileName( stripslashes( $row["sectionImageFileName"] ) );
                    $image->setDescription( stripslashes( $row["sectionImageDescription"] ) );
                    $image->setCreated( new OnpubDateTime( $row["sectionImageCreated"] ) );
                    $image->setModified( new OnpubDateTime( $row["sectionImageModified"] ) );

                    $section->setImage( $image );
                }

                $sections = $website->getSections();
                $exists = false;

                foreach ( $sections as $s ) {
                    if ( $s->getID() == $section->getID() ) {
                        $exists = true;
                    }
                }

                if ( $exists == false ) {
                    $sections[] = $section;
                    $website->setSections( $sections );
                }
            }
        }

        if ( !$joinArticles ) {
            return $website;
        }

        $articlesassoc = array();

        reset( $rows );
        foreach ( $rows as $row ) {
            $articleID = $row["articleID"];
            $sectionID = $row["sectionID"];

            if ( $articleID && $sectionID ) {
                if ( $includeContent ) {
                    $content = stripslashes( $row["content"] );
                }
                else {
                    $content = "";
                }

                $article = new OnpubArticle();

                $article->setID( $articleID );
                $article->setImageID( $row["articleImageID"] );
                $article->setTitle( stripslashes( $row["title"] ) );
                $article->setContent( $content );
                $article->setURL( stripslashes( $row["articleURL"] ) );
                $article->setCreated( new OnpubDateTime( $row["articleCreated"] ) );
                $article->setModified( new OnpubDateTime( $row["articleModified"] ) );

                if ( isset( $articlesassoc["$articleID"] ) ) {
                    $article = $articlesassoc["$articleID"];
                }
                else {
                    $article->appendSectionID( $sectionID );
                    $articlesassoc["$articleID"] = $article;
                }

                if ( $row["articleImageID"] ) {
                    $image = new OnpubImage();

                    $image->setID( $row["articleImageID"] );
                    $image->setWebsiteID( $row["articleImageWebsiteID"] );
                    $image->setFileName( stripslashes( $row["articleImageFileName"] ) );
                    $image->setDescription( stripslashes( $row["articleImageDescription"] ) );
                    $image->setCreated( new OnpubDateTime( $row["articleImageCreated"] ) );
                    $image->setModified( new OnpubDateTime( $row["articleImageModified"] ) );

                    $article->setImage( $image );
                }

                $section = $sectionsassoc["$sectionID"];
                $articles = $section->getArticles();
                $exists = false;

                foreach ( $articles as $a ) {
                    if ( $a->getID() == $article->getID() ) {
                        $exists = true;
                    }
                }

                if ( $exists == false ) {
                    $articles[] = $article;
                    $section->setArticles( $articles );
                }
            }
        }

        if ( $flatArticleList ) {
            $articles = array();

            foreach ( $articlesassoc as $article ) {
                $articles[] = $article;
            }

            $website->setArticles( $articles );
        }

        $authorsassoc = array();

        reset( $rows );
        foreach ( $rows as $row ) {
            $authorID = $row["authorID"];
            $articleID = $row["articleID"];

            if ( $authorID && $articleID ) {
                $author = new OnpubAuthor();

                $author->setID( $authorID );
                $author->setImageID( $row["authorImageID"] );
                $author->setGivenNames( stripslashes( $row["givenNames"] ) );
                $author->setFamilyName( stripslashes( $row["familyName"] ) );
                $author->setDisplayAs( stripslashes( $row["displayAs"] ) );
                $author->setCreated( new OnpubDateTime( $row["authorCreated"] ) );
                $author->setModified( new OnpubDateTime( $row["authorModified"] ) );

                if ( isset( $authorsassoc["$authorID"] ) ) {
                    $author = $authorsassoc["$authorID"];
                }
                else {
                    $authorsassoc["$authorID"] = $author;
                }

                if ( $row["authorImageID"] ) {
                    $image = new OnpubImage();

                    $image->setID( $row["authorImageID"] );
                    $image->setWebsiteID( $row["authorImageWebsiteID"] );
                    $image->setFileName( stripslashes( $row["authorImageFileName"] ) );
                    $image->setDescription( stripslashes( $row["authorImageDescription"] ) );
                    $image->setCreated( new OnpubDateTime( $row["authorImageCreated"] ) );
                    $image->setModified( new OnpubDateTime( $row["authorImageModified"] ) );

                    $author->setImage( $image );
                }

                $article = $articlesassoc["$articleID"];
                $authors = $article->getAuthors();
                $exists = false;

                foreach ( $authors as $a ) {
                    if ( $a->getID() == $author->getID() ) {
                        $exists = true;
                    }
                }

                if ( $exists == false ) {
                    $authors[] = $author;
                    $article->setAuthors( $authors );
                }
            }
        }

        $result->closeCursor();
        return $website;
    }

    private function getQuery( $ID, $join, $includeContent, $orderBy, $order, $dateLimit, $dateAfter, $joinArticles )
    {
        if ( $joinArticles ) {
            if ( $orderBy ) {
                if ( $order ) {
                    $orderBy = "articles.$orderBy $order";
                }
                else {
                    $orderBy = "articles.$orderBy ASC";
                }
            }
            else {
                if ( $order ) {
                    $orderBy = "wsmaps.ID ASC, samaps.ID $order";
                }
                else {
                    $orderBy = "wsmaps.ID ASC, samaps.ID ASC";
                }
            }
        }
        else {
            if ( $orderBy ) {
                if ( $order ) {
                    $orderBy = "wsmaps.$orderBy $order";
                }
                else {
                    $orderBy = "wsmaps.$orderBy ASC";
                }
            }
            else {
                if ( $order ) {
                    $orderBy = "wsmaps.ID $order";
                }
                else {
                    $orderBy = "wsmaps.ID ASC";
                }
            }
        }

        $where = "WHERE websites.ID = $ID";

        if ( $dateLimit && $joinArticles ) {
            $where .= " AND articles.created <= '$dateLimit'";
        }

        if ( $dateAfter && !$dateLimit && $joinArticles ) {
            $where .= " AND articles.created >= '$dateAfter'";
        }

        if ( $includeContent ) {
            $articleColumns = "articles.ID AS articleID, articles.imageID AS articleImageID, title, content, articles.url AS articleURL, articles.created AS articleCreated, articles.modified AS articleModified";
        }
        else {
            $articleColumns = "articles.ID AS articleID, articles.imageID AS articleImageID, title, articles.url AS articleURL, articles.created AS articleCreated, articles.modified AS articleModified";
        }

        if ( $join ) {
            if ( $joinArticles ) {
                return "SELECT websites.ID, websites.imageID, websites.name, websites.url, websites.imagesURL, websites.imagesDirectory, websites.created, websites.modified, websiteimages.fileName, websiteimages.description, websiteimages.created AS imageCreated, websiteimages.modified AS imageModified, sections.ID AS sectionID, sections.imageID AS sectionImageID, sections.websiteID AS sectionWebsiteID, sections.name AS sectionName, sections.created AS sectionCreated, sections.modified AS sectionModified, sectionimages.websiteID AS sectionImageWebsiteID, sectionimages.fileName AS sectionImageFileName, sectionimages.description AS sectionImageDescription, sectionimages.created AS sectionImageCreated, sectionimages.modified AS sectionImageModified, $articleColumns, articleimages.websiteID AS articleImageWebsiteID, articleimages.fileName AS articleImageFileName, articleimages.description AS articleImageDescription, articleimages.created AS articleImageCreated, articleimages.modified AS articleImageModified, authors.ID AS authorID, authors.imageID AS authorImageID, authors.givenNames, authors.familyName, authors.displayAs, authors.created AS authorCreated, authors.modified AS authorModified, authorimages.websiteID AS authorImageWebsiteID, authorimages.fileName AS authorImageFileName, authorimages.description AS authorImageDescription, authorimages.created AS authorImageCreated, authorimages.modified AS authorImageModified FROM OnpubWebsites AS websites LEFT JOIN OnpubImages AS websiteimages ON websites.imageID = websiteimages.ID LEFT JOIN OnpubWSMaps AS wsmaps ON websites.ID = wsmaps.websiteID LEFT JOIN OnpubSections AS sections ON wsmaps.sectionID = sections.ID LEFT JOIN OnpubImages AS sectionimages ON sections.imageID = sectionimages.ID LEFT JOIN OnpubSAMaps AS samaps ON sections.ID = samaps.sectionID LEFT JOIN OnpubArticles AS articles ON samaps.articleID = articles.ID LEFT JOIN OnpubImages AS articleimages ON articles.imageID = articleimages.ID LEFT JOIN OnpubAAMaps AS aamaps ON articles.ID = aamaps.articleID LEFT JOIN OnpubAuthors AS authors ON aamaps.authorID = authors.ID LEFT JOIN OnpubImages AS authorimages ON authors.imageID = authorimages.ID $where ORDER BY $orderBy";
            }
            else {
                return "SELECT websites.ID, websites.imageID, websites.name, websites.url, websites.imagesURL, websites.imagesDirectory, websites.created, websites.modified, websiteimages.fileName, websiteimages.description, websiteimages.created AS imageCreated, websiteimages.modified AS imageModified, sections.ID AS sectionID, sections.imageID AS sectionImageID, sections.websiteID AS sectionWebsiteID, sections.name AS sectionName, sections.created AS sectionCreated, sections.modified AS sectionModified, sectionimages.websiteID AS sectionImageWebsiteID, sectionimages.fileName AS sectionImageFileName, sectionimages.description AS sectionImageDescription, sectionimages.created AS sectionImageCreated, sectionimages.modified AS sectionImageModified FROM OnpubWebsites AS websites LEFT JOIN OnpubImages AS websiteimages ON websites.imageID = websiteimages.ID LEFT JOIN OnpubWSMaps AS wsmaps ON websites.ID = wsmaps.websiteID LEFT JOIN OnpubSections AS sections ON wsmaps.sectionID = sections.ID LEFT JOIN OnpubImages AS sectionimages ON sections.imageID = sectionimages.ID $where ORDER BY $orderBy";
            }
        }
        else {
            return "SELECT ID, imageID, name, url, imagesURL, imagesDirectory, created, modified FROM OnpubWebsites WHERE ID = $ID";
        }
    }

    /**
     * @param OnpubWebsite $website Website whose ID you want to get.
     * @return int The ID of the website. NULL if the website does not exist in the database.
     */
    public function getID( OnpubWebsite $website )
    {
        $stmt = $this->pdo->prepare( "SELECT ID FROM OnpubWebsites WHERE name = :name" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );

        $stmt->bindParam( ':name', OnpubDatabase::utf8Decode( trim( $website->getName() ) ) );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, false, $stmt->errorInfo() );

        if ( ($row = $stmt->fetch( PDO::FETCH_ASSOC )) ) {
            return $row["ID"];
        }

        return null;
    }

    /**
     * @param mixed $websites A single {@link OnpubWebsite} object or an array of {@link OnpubWebsite} objects (to insert multiple websites at a time).
     * @return mixed The ID(s) of the new website(s). An int will be returned if a single website was inserted. An array of ints will be returned if multiple websites were inserted.
     * @throws PDOException if there's a database error.
     */
    public function insert( $websites )
    {
        $oimages = new OnpubImages( $this->pdo, false );
        $IDs = array();
        $isArray = true;

        if ( !is_array( $websites ) ) {
            $websites = array( $websites );
            $isArray = false;
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        $stmt = $this->pdo->prepare( "INSERT INTO OnpubWebsites (ID, imageID, name, url, imagesURL, imagesDirectory, created, modified) VALUES (:ID, :imageID, :name, :url, :imagesURL, :imagesDirectory, :created, :modified)" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        foreach ( $websites as $website ) {
            if ( $website->getImage() ) {
                try {
                    $imageID = $oimages->insert( $website->getImage() );
                    $website->setImageID( $imageID );
                }
                catch ( PDOException $e ) {
                    if ( $this->enableTransactions ) $this->pdo->rollBack();
                    throw $e;
                }
            }

            try {
                $ID = $this->getID( $website );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }

            if ( $ID ) {
                $IDs[] = $ID;
                $website->setID( $ID );
            }
            else {
                $stmt->bindParam( ':ID', $website->getID() );
                $stmt->bindParam( ':imageID', $website->getImageID() );
                $stmt->bindParam( ':name', OnpubDatabase::utf8Decode( trim( $website->getName() ) ) );
                $stmt->bindParam( ':url', OnpubDatabase::utf8Decode( trim( $website->getURL() ) ) );
                $stmt->bindParam( ':imagesURL', OnpubDatabase::utf8Decode( trim( $website->getImagesURL() ) ) );
                $stmt->bindParam( ':imagesDirectory', OnpubDatabase::utf8Decode( trim( $website->getImagesDirectory() ) ) );
                $stmt->bindParam( ':created', $website->getCreated()->toString() );
                $stmt->bindParam( ':modified', $website->getModified()->toString() );
                $result = $stmt->execute();
                OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

                $IDs[] = $this->pdo->lastInsertId();
                $website->setID( $this->pdo->lastInsertId() );
            }
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        if ( $isArray ) {
            return $IDs;
        }
        else {
            return end( $IDs );
        }
    }

    /**
     * @param OnpubWebsite $website The website to be updated.
     * @return int 1 if the website was updated. 0 if the website does not exist in the database.
     */
    public function update( OnpubWebsite $website )
    {
        $owsmaps = new OnpubWSMaps( $this->pdo, false );
        $now = new OnpubDateTime();

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        $stmt = $this->pdo->prepare( "UPDATE OnpubWebsites SET imageID = :imageID, name = :name, url = :url, imagesURL = :imagesURL, imagesDirectory = :imagesDirectory, modified = :modified WHERE ID = :ID" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        $stmt->bindParam( ':imageID', $website->getImageID() );
        $stmt->bindParam( ':name', OnpubDatabase::utf8Decode( trim( $website->getName() ) ) );
        $stmt->bindParam( ':url', OnpubDatabase::utf8Decode( trim( $website->getURL() ) ) );
        $stmt->bindParam( ':imagesURL', OnpubDatabase::utf8Decode( trim( $website->getImagesURL() ) ) );
        $stmt->bindParam( ':imagesDirectory', OnpubDatabase::utf8Decode( trim( $website->getImagesDirectory() ) ) );
        $stmt->bindParam( ':modified', $now->toString() );
        $stmt->bindParam( ':ID', $website->getID() );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

        try {
            $owsmaps->delete( $website->getID(), null );
        }
        catch ( PDOException $e ) {
            if ( $this->enableTransactions ) $this->pdo->rollBack();
            throw $e;
        }

        $sections = $website->getSections();
        $wsmaps = array();

        foreach ( $sections as $section ) {
            $wsmap = new OnpubWSMap();

            $wsmap->setWebsiteID( $section->getWebsiteID() );
            $wsmap->setSectionID(  $section->getID() );

            $wsmaps[] = $wsmap;
        }

        try {
            $owsmaps->insert( $wsmaps );
        }
        catch ( PDOException $e ) {
            if ( $this->enableTransactions ) $this->pdo->rollBack();
            throw $e;
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyStatus( $this->pdo, $status );
        }

        return $stmt->rowCount();
    }
}

?>
